searchNodes=[{"doc":"Components are a mechanism to compartmentalize state, markup, and events in LiveView.Components are defined by using Phoenix.LiveComponent and are used by calling Phoenix.LiveView.Helpers.live_component/3 in a parent LiveView. Components run inside the LiveView process, but may have their own state and event handling.The simplest component only needs to define a render function:defmodule HeroComponent do # If you generated an app with mix phx.new --live, # the line below would be: use MyAppWeb, :live_component use Phoenix.LiveComponent def render(assigns) do ~L&quot;&quot;&quot; &lt;div class=&quot;hero&quot;&gt;&lt;%= @content %&gt;&lt;/div&gt; &quot;&quot;&quot; end endWhen use Phoenix.LiveComponent is used, all functions in Phoenix.LiveView are imported. A component can be invoked as:&lt;%= live_component @socket, HeroComponent, content: @content %&gt;Components come in two shapes, stateless or stateful. The component above is a stateless component. Of course, the component above is not any different compared to a regular function. However, as we will see, components do provide their own exclusive feature set.Stateless components life-cycleWhen live_component is called, the following callbacks will be invoked in the component:mount(socket) -&gt; update(assigns, socket) -&gt; render(assigns)First mount/1 is called only with the socket. mount/1 can be used to set any initial state. Then update/2 is invoked with all of the assigns given to live_component/3. The default implementation of update/2 simply merges all assigns into the socket. Then, after the component is updated, render/1 is called with all assigns.A stateless component is always mounted, updated, and rendered whenever the parent template changes. That's why they are stateless: no state is kept after the component.However, any component can be made stateful by passing an :id assign.Stateful components life-cycleA stateful component is a component that receives an :id on live_component/3:&lt;%= live_component @socket, HeroComponent, id: :hero, content: @content %&gt;Stateful components are identified by the component module and their ID. Therefore, two different component modules with the same ID are different components. This means we can often tie the component ID to some application based ID:&lt;%= live_component @socket, UserComponent, id: @user.id, user: @user %&gt;Also note the given :id is not necessarily used as the DOM ID. If you want to set a DOM ID, it is your responsibility to set it when rendering:defmodule UserComponent do use Phoenix.LiveComponent def render(assigns) do ~L&quot;&quot;&quot; &lt;div id=&quot;user-&lt;%= @id %&gt;&quot; class=&quot;user&quot;&gt;&lt;%= @user.name %&gt;&lt;/div&gt; &quot;&quot;&quot; end endIn stateful components, mount/1 is called only once, when the component is first rendered. For each rendering, the optional preload/1 and update/2 callbacks are called before render/1.So on first render, the following callbacks will be invoked:preload(list_of_assigns) -&gt; mount(socket) -&gt; update(assigns, socket) -&gt; render(assigns)On subsequent renders, these callbacks will be invoked:preload(list_of_assigns) -&gt; update(assigns, socket) -&gt; render(assigns)Targeting Component EventsStateful components can also implement the handle_event/3 callback that works exactly the same as in LiveView. For a client event to reach a component, the tag must be annotated with a phx-target. If you want to send the event to yourself, you can simply use the @myself assign, which is an internal unique reference to the component instance:&lt;a href=&quot;#&quot; phx-click=&quot;say_hello&quot; phx-target=&quot;&lt;%= @myself %&gt;&quot;&gt; Say hello! &lt;/a&gt;Note @myself is not set for stateless components, as they cannot receive events.If you want to target another component, you can also pass an ID or a class selector to any element inside the targeted component. For example, if there is a UserComponent with the DOM ID of user-13, using a query selector, we can send an event to it with:&lt;a href=&quot;#&quot; phx-click=&quot;say_hello&quot; phx-target=&quot;#user-13&quot;&gt; Say hello! &lt;/a&gt;In both cases, handle_event/3 will be called with the &quot;say_hello&quot; event. When handle_event/3 is called for a component, only the diff of the component is sent to the client, making them extremely efficient.Any valid query selector for phx-target is supported, provided that the matched nodes are children of a LiveView or LiveComponent, for example to send the close event to multiple components:&lt;a href=&quot;#&quot; phx-click=&quot;close&quot; phx-target=&quot;#modal, #sidebar&quot;&gt; Dismiss &lt;/a&gt;Preloading and updateEvery time a stateful component is rendered, both preload/1 and update/2 are called. To understand why both callbacks are necessary, imagine that you implement a component and the component needs to load some state from the database. For example:&lt;%= live_component @socket, UserComponent, id: user_id %&gt;A possible implementation would be to load the user on the update/2 callback:def update(assigns, socket) do user = Repo.get! User, assigns.id {:ok, assign(socket, :user, user)} endHowever, the issue with said approach is that, if you are rendering multiple user components in the same page, you have a N+1 query problem. The preload/1 callback helps address this problem as it is invoked with a list of assigns for all components of the same type. For example, instead of implementing update/2 as above, one could implement:def preload(list_of_assigns) do list_of_ids = Enum.map(list_of_assigns, &amp; &amp;1.id) users = from(u in User, where: u.id in ^list_of_ids, select: {u.id, u}) |&gt; Repo.all() |&gt; Map.new() Enum.map(list_of_assigns, fn assigns -&gt; Map.put(assigns, :user, users[assigns.id]) end) endNow only a single query to the database will be made. In fact, the preloading algorithm is a breadth-first tree traversal, which means that even for nested components, the amount of queries are kept to a minimum.Finally, note that preload/1 must return an updated list_of_assigns, keeping the assigns in the same order as they were given.Managing stateNow that we have learned how to define and use components, as well as how to use preload/1 as a data loading optimization, it is important to talk about how to manage state in components.Generally speaking, you want to avoid both the parent LiveView and the LiveComponent working on two different copies of the state. Instead, you should assume only one of them to be the source of truth. Let's discuss the two different approaches in detail.Imagine a scenario where LiveView represents a board with each card in it as a separate component. Each card has a form that allows to update its title directly in the component. We will see how to organize the data flow keeping either the view or the component as the source of truth.LiveView as the source of truthIf the LiveView is the source of truth, it will be responsible for fetching all of the cards in a board. Then it will call live_component/3 for each card, passing the card struct as argument to CardComponent:&lt;%= for card &lt;- @cards do %&gt; &lt;%= live_component @socket, CardComponent, card: card, id: card.id, board_id: @id %&gt; &lt;% end %&gt;Now, when the user submits a form inside the CardComponent to update the card, CardComponent.handle_event/3 will be triggered. However, if the update succeeds, you must not change the card struct inside the component. If you do so, the card struct in the component will get out of sync with the LiveView. Since the LiveView is the source of truth, you should instead tell the LiveView that the card was updated.Luckily, because the component and the view run in the same process, sending a message from the component to the parent LiveView is as simple as sending a message to self():defmodule CardComponent do ... def handle_event(&quot;update_title&quot;, %{&quot;title&quot; =&gt; title}, socket) do send self(), {:updated_card, %{socket.assigns.card | title: title}} {:noreply, socket} end endThe LiveView then receives this event using handle_info:defmodule BoardView do ... def handle_info({:updated_card, card}, socket) do # update the list of cards in the socket {:noreply, updated_socket} end endAs the list of cards in the parent socket was updated, the parent will be re-rendered, sending the updated card to the component. So in the end, the component does get the updated card, but always driven from the parent.Alternatively, instead of having the component directly send a message to the parent, the component could broadcast the update using Phoenix.PubSub. Such as:defmodule CardComponent do ... def handle_event(&quot;update_title&quot;, %{&quot;title&quot; =&gt; title}, socket) do message = {:updated_card, %{socket.assigns.card | title: title}} Phoenix.PubSub.broadcast(MyApp.PubSub, board_topic(socket), message) {:noreply, socket} end defp board_topic(socket) do &quot;board:&quot; &lt;&gt; socket.assigns.board_id end endAs long as the parent LiveView subscribes to the &quot;board:ID&quot; topic, it will receive updates. The advantage of using PubSub is that we get distributed updates out of the box. Now, if any user connected to the board changes a card, all other users will see the change.LiveComponent as the source of truthIf the component is the source of truth, then the LiveView must no longer fetch all of the cards structs from the database. Instead, the view must only fetch all of the card ids and render the component only by passing the IDs:&lt;%= for card_id &lt;- @card_ids do %&gt; &lt;%= live_component @socket, CardComponent, id: card_id, board_id: @id %&gt; &lt;% end %&gt;Now, each CardComponent loads their own card. Of course, doing so per card would be expensive and lead to N queries, where N is the number of components, so we must use the preload/1 callback to make it efficient.Once all card components are started, they can fully manage each card as a whole, without concerning themselves with the parent LiveView.However, note that components do not have a handle_info/2 callback. Therefore, if you want to track distributed changes on a card, you must have the parent LiveView receive those events and redirect them to the appropriate card. For example, assuming card updates are sent to the &quot;board:ID&quot; topic, and that the board LiveView is subscribed to said topic, one could do:def handle_info({:updated_card, card}, socket) do send_update CardComponent, id: card.id, board_id: socket.assigns.id {:noreply, socket} endWith send_update, the CardComponent given by id will be invoked, triggering both preload and update callbacks, which will load the most up to date data from the database.Live component blocksWhen live_component is invoked, it is also possible to pass a do/end block:&lt;%= live_component @socket, GridComponent, entries: @entries do %&gt; New entry: &lt;%= @entry %&gt; &lt;% end %&gt;The do/end will be available in an assign named @inner_block. You can render its contents by calling render_block with the assign itself and a keyword list of assigns to inject into the rendered content. For example, the grid component above could be implemented as:defmodule GridComponent do use Phoenix.LiveComponent def render(assigns) do ~L&quot;&quot;&quot; &lt;div class=&quot;grid&quot;&gt; &lt;%= for entry &lt;- @entries do %&gt; &lt;div class=&quot;column&quot;&gt; &lt;%= render_block(@inner_block, entry: entry) %&gt; &lt;/div&gt; &lt;% end %&gt; &lt;/div&gt; &quot;&quot;&quot; end endWhere the :entry assign was injected into the do/end block.Note the @inner_block assign is also passed to update/2 along all other assigns. So if you have a custom update/2 implementation, make sure to assign it to the socket like so:def update(%{inner_block: inner_block}, socket) do {:ok, assign(socket, inner_block: inner_block)} endThe above approach is the preferred one when passing blocks to do/end. However, if you are outside of a .leex template and you want to invoke a component passing a do/end block, you will have to explicitly handle the assigns by giving it a -&gt; clause:live_component @socket, GridComponent, entries: @entries do new_assigns -&gt; &quot;New entry: &quot; &lt;&gt; new_assigns[:entry] endLive patches and live redirectsA template rendered inside a component can use live_patch and live_redirect calls. The live_patch is always handled by the parent LiveView, as components do not provide handle_params.Cost of stateful componentsThe internal infrastructure LiveView uses to keep track of stateful components is very lightweight. However, be aware that in order to provide change tracking and to send diffs over the wire, all of the components assigns are kept in memory - exactly as it is done in LiveViews themselves.Therefore it is your responsibility to keep only the assigns necessary in each component. For example, avoid passing all of LiveView's assigns when rendering a component:&lt;%= live_component @socket, MyComponent, assigns %&gt;Instead pass only the keys that you need:&lt;%= live_component @socket, MyComponent, user: @user, org: @org %&gt;Luckily, because LiveViews and LiveComponents are in the same process, they share the same data structures. For example, in the code above, the view and the component will share the same copies of the @user and @org assigns.You should also avoid using stateful components to provide abstract DOM components. As a guideline, a good LiveComponent encapsulates application concerns and not DOM functionality. For example, if you have a page that shows products for sale, you can encapsulate the rendering of each of those products in a component. This component may have many buttons and events within it. On the opposite side, do not write a component that is simply encapsulating generic DOM components. For instance, do not do this:defmodule MyButton do use Phoenix.LiveComponent def render(assigns) do ~L&quot;&quot;&quot; &lt;button class=&quot;css-framework-class&quot; phx-click=&quot;click&quot;&gt; &lt;%= @text %&gt; &lt;/button&gt; &quot;&quot;&quot; end def handle_event(&quot;click&quot;, _, socket) do _ = socket.assigns.on_click.() {:noreply, socket} end endInstead, it is much simpler to create a function:def my_button(text, click) do assigns = %{text: text, click: click} ~L&quot;&quot;&quot; &lt;button class=&quot;css-framework-class&quot; phx-click=&quot;&lt;%= @click %&gt;&quot;&gt; &lt;%= @text %&gt; &lt;/button&gt; &quot;&quot;&quot; endIf you keep components mostly as an application concern with only the necessary assigns, it is unlikely you will run into issues related to stateful components.LimitationsComponents require at least one HTML tagComponents must only contain HTML tags at their root. At least one HTML tag must be present. It is not possible to have components that render only text or text mixed with tags at the root.Change tracking requirementAnother limitation of components is that they must always be change tracked. For example, if you render a component inside form_for, like this:&lt;%= form_for @changeset, &quot;#&quot;, fn f -&gt; %&gt; &lt;%= live_component @socket, SomeComponent, f: f %&gt; &lt;% end %&gt;The component ends up enclosed by the form markup, where LiveView cannot track it. In such cases, you may receive an error such as:** (ArgumentError) cannot convert component SomeComponent to HTML. A component must always be returned directly as part of a LiveView templateIn this particular case, this can be addressed by using the form_for variant without anonymous functions:&lt;%= f = form_for @changeset, &quot;#&quot; %&gt; &lt;%= live_component @socket, SomeComponent, f: f %&gt; &lt;/form&gt;This issue can also happen with other helpers, such as content_tag:&lt;%= content_tag :div do %&gt; &lt;%= live_component @socket, SomeComponent, f: f %&gt; &lt;% end %&gt;In this case, the solution is to not use content_tag and rely on LiveEEx to build the markup.SVG supportGiven components compartmentalize markup on the server, they are also rendered in isolation on the client, which provides great performance benefits on the client too.However, when rendering components on the client, the client needs to choose the mime type of the component contents, which defaults to HTML. This is the best default but in some cases it may lead to unexpected results.For example, if you are rendering SVG, the SVG will be interpreted as HTML. This may work just fine for most components but you may run into corner cases. For example, the &lt;image&gt; SVG tag may be rewritten to the &lt;img&gt; tag, since &lt;image&gt; is an obsolete HTML tag.Luckily, there is a solution to this problem. Since SVG allows &lt;svg&gt; tags to be nested, you can wrap the component content into an &lt;svg&gt; tag. This will ensure that it is correctly interpreted by the browser.","ref":"Phoenix.LiveComponent.html","title":"Phoenix.LiveComponent","type":"behaviour"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:handle_event/3","title":"Phoenix.LiveComponent.handle_event/3","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:mount/1","title":"Phoenix.LiveComponent.mount/1","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:preload/1","title":"Phoenix.LiveComponent.preload/1","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:render/1","title":"Phoenix.LiveComponent.render/1","type":"callback"},{"doc":"","ref":"Phoenix.LiveComponent.html#c:update/2","title":"Phoenix.LiveComponent.update/2","type":"callback"},{"doc":"The struct representing an internal unique reference to the component instance, available as the @myself assign in stateful components.Read more about the uses of @myself in the Phoenix.LiveComponent docs.","ref":"Phoenix.LiveComponent.CID.html","title":"Phoenix.LiveComponent.CID","type":"module"},{"doc":"LiveView provides rich, real-time user experiences with server-rendered HTML.The LiveView programming model is declarative: instead of saying &quot;once event X happens, change Y on the page&quot;, events in LiveView are regular messages which may cause changes to its state. Once the state changes, LiveView will re-render the relevant parts of its HTML template and push it to the browser, which updates itself in the most efficient manner. This means developers write LiveView templates as any other server-rendered HTML and LiveView does the hard work of tracking changes and sending the relevant diffs to the browser.At the end of the day, a LiveView is nothing more than a process that receives events as messages and updates its state. The state itself is nothing more than functional and immutable Elixir data structures. The events are either internal application messages (usually emitted by Phoenix.PubSub) or sent by the client/browser.LiveView is first rendered statically as part of regular HTTP requests, which provides quick times for &quot;First Meaningful Paint&quot;, in addition to helping search and indexing engines. Then a persistent connection is established between client and server. This allows LiveView applications to react faster to user events as there is less work to be done and less data to be sent compared to stateless requests that have to authenticate, decode, load, and encode data on every request. The flipside is that LiveView uses more memory on the server compared to stateless requests.Use casesThere are many use cases where LiveView is an excellent fit right now:Handling of user interaction and inputs, buttons, and forms - such as input validation, dynamic forms, autocomplete, etc;Events and updates pushed by server - such as notifications, dashboards, etc;Page and data navigation - such as navigating between pages, pagination, etc can be built with LiveView using the excellent live navigation feature set. This reduces the amount of data sent over the wire, gives developers full control over the LiveView life-cycle, while controlling how the browser tracks those changes in state;There are also use cases which are a bad fit for LiveView:Animations - animations, menus, and general UI events that do not need the server in the first place are a bad fit for LiveView. Those can be achieved without LiveView in multiple ways, such as with CSS and CSS transitions, using LiveView hooks, or even integrating with UI toolkits designed for this purpose, such as Bootstrap, Alpine.JS, and similar.Life-cycleA LiveView begins as a regular HTTP request and HTML response, and then upgrades to a stateful view on client connect, guaranteeing a regular HTML page even if JavaScript is disabled. Any time a stateful view changes or updates its socket assigns, it is automatically re-rendered and the updates are pushed to the client.You begin by rendering a LiveView typically from your router. When LiveView is first rendered, the mount/3 callback is invoked with the current params, the current session and the LiveView socket. As in a regular request, params contains public data that can be modified by the user. The session always contains private data set by the application itself. The mount/3 callback wires up socket assigns necessary for rendering the view. After mounting, render/1 is invoked and the HTML is sent as a regular HTML response to the client.After rendering the static page, LiveView connects from the client to the server where stateful views are spawned to push rendered updates to the browser, and receive client events via phx- bindings. Just like the first rendering, mount/3 is invoked with params, session, and socket state, where mount assigns values for rendering. However in the connected client case, a LiveView process is spawned on the server, pushes the result of render/1 to the client and continues on for the duration of the connection. If at any point during the stateful life-cycle a crash is encountered, or the client connection drops, the client gracefully reconnects to the server, calling mount/3 once again.ExampleBefore writing your first example, make sure that Phoenix LiveView is properly installed. If you are just getting started, this can be easily done by running mix phx.new my_app --live. The phx.new command with the --live flag will create a new project with LiveView installed and configured. Otherwise, please follow the steps in the installation guide before continuing.A LiveView is a simple module that requires two callbacks: mount/3 and render/1:defmodule MyAppWeb.ThermostatLive do # If you generated an app with mix phx.new --live, # the line below would be: use MyAppWeb, :live_view use Phoenix.LiveView def render(assigns) do ~L&quot;&quot;&quot; Current temperature: &lt;%= @temperature %&gt; &quot;&quot;&quot; end def mount(_params, %{&quot;current_user_id&quot; =&gt; user_id}, socket) do temperature = Thermostat.get_user_reading(user_id) {:ok, assign(socket, :temperature, temperature)} end endThe render/1 callback receives the socket.assigns and is responsible for returning rendered content. You can use Phoenix.LiveView.Helpers.sigil_L/2 to inline LiveView templates.Next, decide where you want to use your LiveView.You can serve the LiveView directly from your router (recommended):defmodule MyAppWeb.Router do use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot;, MyAppWeb do live &quot;/thermostat&quot;, ThermostatLive end endNote: the above assumes there is plug :put_root_layout call in your router that configures the LiveView layout. This call is automatically included by mix phx.new --live and described in the installation guide. If you don't want to configure a root layout, you must pass layout: {MyAppWeb.LayoutView, &quot;app.html&quot;} as an option to the live macro above.Alternatively, you can live_render from any template:&lt;h1&gt;Temperature Control&lt;/h1&gt; &lt;%= live_render(@conn, MyAppWeb.ThermostatLive) %&gt;Or you can live_render your view from any controller:defmodule MyAppWeb.ThermostatController do ... import Phoenix.LiveView.Controller def show(conn, %{&quot;id&quot; =&gt; id}) do live_render(conn, MyAppWeb.ThermostatLive) end endWhen a LiveView is rendered, all of the data currently stored in the connection session (see Plug.Conn.get_session/1) will be given to the LiveView.It is also possible to pass additional session information to the LiveView through a session parameter:# In the router live &quot;/thermostat&quot;, ThermostatLive, session: %{&quot;extra_token&quot; =&gt; &quot;foo&quot;} # In a view &lt;%= live_render(@conn, MyAppWeb.ThermostatLive, session: %{&quot;extra_token&quot; =&gt; &quot;foo&quot;}) %&gt;Notice the :session uses string keys as a reminder that session data is serialized and sent to the client. So you should always keep the data in the session to a minimum. For example, instead of storing a User struct, you should store the &quot;user_id&quot; and load the User when the LiveView mounts.Once the LiveView is rendered, a regular HTML response is sent. In your app.js file, you should find the following:import {Socket} from &quot;phoenix&quot; import LiveSocket from &quot;phoenix_live_view&quot; let csrfToken = document.querySelector(&quot;meta[name=&#39;csrf-token&#39;]&quot;).getAttribute(&quot;content&quot;) let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {params: {_csrf_token: csrfToken}}) liveSocket.connect()After the client connects, mount/3 will be invoked inside a spawned LiveView process. At this point, you can use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc. For example, you can periodically update a LiveView with a timer:defmodule DemoWeb.ThermostatLive do use Phoenix.LiveView ... def mount(_params, %{&quot;current_user_id&quot; =&gt; user_id}, socket) do if connected?(socket), do: Process.send_after(self(), :update, 30000) case Thermostat.get_user_reading(user_id) do {:ok, temperature} -&gt; {:ok, assign(socket, temperature: temperature, user_id: user_id)} {:error, _reason} -&gt; {:ok, redirect(socket, to: &quot;/error&quot;)} end end def handle_info(:update, socket) do Process.send_after(self(), :update, 30000) {:ok, temperature} = Thermostat.get_reading(socket.assigns.user_id) {:noreply, assign(socket, :temperature, temperature)} end endWe used connected?(socket) on mount to send our view a message every 30s if the socket is in a connected state. We receive the :update message in the handle_info/2 callback, just like in an Elixir GenServer, and update our socket assigns. Whenever a socket's assigns change, render/1 is automatically invoked, and the updates are sent to the client.Colocating templatesIn the examples above, we have placed the template directly inside the LiveView:defmodule MyAppWeb.ThermostatLive do use Phoenix.LiveView def render(assigns) do ~L&quot;&quot;&quot; Current temperature: &lt;%= @temperature %&gt; &quot;&quot;&quot; endFor larger templates, you can place them in a file in the same directory and same name as the LiveView. For example, if the file above is placed at lib/my_app_web/live/thermostat_live.ex, you can also remove the render/1 definition above and instead put the template code at lib/my_app_web/live/thermostat_live.html.leex.Alternatively, you can keep the render/1 callback but delegate to an existing Phoenix.View module in your application. For example:defmodule MyAppWeb.ThermostatLive do use Phoenix.LiveView def render(assigns) do Phoenix.View.render(MyAppWeb.PageView, &quot;page.html&quot;, assigns) end endIn all cases, each assign in the template will be accessible as @assign. You can learn more about assigns and LiveEEx templates in their own guide.BindingsPhoenix supports DOM element bindings for client-server interaction. For example, to react to a click on a button, you would render the element:&lt;button phx-click=&quot;inc_temperature&quot;&gt;+&lt;/button&gt;Then on the server, all LiveView bindings are handled with the handle_event callback, for example:def handle_event(&quot;inc_temperature&quot;, _value, socket) do {:ok, new_temp} = Thermostat.inc_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} endBindingAttributesParamsphx-value-*Click Eventsphx-capture-click, phx-clickFocus/Blur Eventsphx-window-focus, phx-window-blur, phx-focus, phx-blurKey Eventsphx-key, phx-window-keyup, phx-window-keydown, phx-keyup, phx-keydownForm Eventsphx-auto-recover, phx-trigger-action, phx-disable-with, phx-feedback-for, phx-submit, phx-changeRate Limitingphx-throttle, phx-debounceDOM Patchingphx-updateJS Interopphx-hookCompartmentalizing markup and events with render, live_render, and live_componentWe can render another template directly from a LiveView template by simply calling render:render SomeView, &quot;child_template.html&quot;, assignsWhere SomeView is a regular Phoenix.View, typically defined in lib/my_app_web/views/some_view.ex and &quot;child_template.html&quot; is defined at lib/my_app_web/templates/some_view/child_template.html.leex. As long as the template has the .leex extension and all assigns are passed, LiveView change tracking will also work across templates.When rendering a child template, any of the phx-* events in the child template will be sent to the LiveView. In other words, similar to regular Phoenix templates, a regular render call does not start another LiveView. This means render is useful for sharing markup between views.If you want to start a separate LiveView from within a LiveView, then you can call live_render/3 instead of render/3. This child LiveView runs in a separate process than the parent, with its own mount and handle_event callbacks. If a child LiveView crashes, it won't affect the parent. If the parent crashes, all children are terminated.When rendering a child LiveView, the :id option is required to uniquely identify the child. A child LiveView will only ever be rendered and mounted a single time, provided its ID remains unchanged. Updates to a child session will be merged on the client, but not passed back up until either a crash and re-mount or a connection drop and recovery. To force a child to re-mount with new session data, a new ID must be provided.Given that a LiveView runs on its own process, it is an excellent tool for creating completely isolated UI elements, but it is a slightly expensive abstraction if all you want is to compartmentalize markup and events. For example, if you are showing a table with all users in the system, and you want to compartmentalize this logic, rendering a separate LiveView for each user, then using a process per user would likely be too expensive. For these cases, LiveView provides Phoenix.LiveComponent, which are rendered using live_component/3:&lt;%= live_component(@socket, UserComponent, id: user.id, user: user) %&gt;Components have their own mount and handle_event callbacks, as well as their own state with change tracking support. Components are also lightweight as they &quot;run&quot; in the same process as the parent LiveView. However, this means an error in a component would cause the whole view to fail to render. See Phoenix.LiveComponent for a complete rundown on components.To sum it up:render - compartmentalizes markuplive_component - compartmentalizes state, markup, and eventslive_render - compartmentalizes state, markup, events, and error isolationEndpoint configurationLiveView accepts the following configuration in your endpoint under the :live_view key::signing_salt (required) - the salt used to sign data sent to the client:hibernate_after (optional) - the idle time in milliseconds allowed in the LiveView before compressing its own memory and state. Defaults to 15000ms (15 seconds)GuidesLiveView has many guides to help you on your journey.Server-sideThese guides focus on server-side functionality:Assigns and LiveEExError and exception handlingLive LayoutsLive NavigationSecurity considerations of the LiveView modelTelemetryUploadsUsing Gettext for internationalizationClient-sideThese guides focus on LiveView bindings and client-side integration:BindingsForm bindingsDOM patching and temporary assignsJavaScript interoperabilityUploads (External)","ref":"Phoenix.LiveView.html","title":"Phoenix.LiveView","type":"behaviour"},{"doc":"Uses LiveView in the current module to mark it a LiveView.use Phoenix.LiveView, namespace: MyAppWeb, container: {:tr, class: &quot;colorized&quot;}, layout: {MyAppWeb.LayoutView, &quot;live.html&quot;}Options:namespace - configures the namespace the LiveView is in:container - configures the container the LiveView will be wrapped in:layout - configures the layout the LiveView will be rendered in","ref":"Phoenix.LiveView.html#__using__/1","title":"Phoenix.LiveView.__using__/1","type":"macro"},{"doc":"Allows an upload for the provided name.Options:accept - Required. A list of unique file type specifiers or the atom :any to allow any kind of file. For example, [&quot;.jpeg&quot;], :any, etc.:max_entries - The maximum number of selected files to allow per file input. Defaults to 1.:max_file_size - The maximum file size in bytes to allow to be uploaded. Defaults 8MB. For example, 12_000_000.:chunk_size - The chunk size in bytes to send when uploading. Defaults 64_000.:chunk_timeout - The time in milliseconds to wait before closing the upload channel when a new chunk has not been received. Defaults 10_000.:external - The 2-arity function for generating metadata for external client uploaders. See the Uploads section for example usage.:progress - The optional 3-arity function for receiving progress events:auto_upload - Instructs the client to upload the file automatically on file selection instead of waiting for form submits. Default false.Raises when a previously allowed upload under the same name is still active.Examplesallow_upload(socket, :avatar, accept: ~w(.jpg .jpeg), max_entries: 2) allow_upload(socket, :avatar, accept: :any)For consuming files automatically as they are uploaded, you can pair auto_upload: true with a custom progress function to consume the entries as they are completed. For example:allow_upload(socket, :avatar, accept: :any, progress: &amp;handle_progress/3, auto_upload: true) defp handle_progress(:avatar, entry, socket) do if entry.done? do uploaded_file = consume_uploaded_entry(socket, entry, fn %{} = meta -&gt; ... end) {:noreply, put_flash(socket, :info, &quot;file \#{uploaded_file.name} uploaded&quot;)} else {:noreply, socket} end end","ref":"Phoenix.LiveView.html#allow_upload/3","title":"Phoenix.LiveView.allow_upload/3","type":"function"},{"doc":"See assign/3.","ref":"Phoenix.LiveView.html#assign/2","title":"Phoenix.LiveView.assign/2","type":"function"},{"doc":"Adds key value pairs to socket assigns.A single key value pair may be passed, or a keyword list or a map of assigns may be provided to be merged into existing socket assigns.Examplesiex&gt; assign(socket, :name, &quot;Elixir&quot;) iex&gt; assign(socket, name: &quot;Elixir&quot;, logo: &quot;💧&quot;) iex&gt; assign(socket, %{name: &quot;Elixir&quot;})","ref":"Phoenix.LiveView.html#assign/3","title":"Phoenix.LiveView.assign/3","type":"function"},{"doc":"Assigns a value into the socket only if it does not exist.Useful for lazily assigning values and referencing parent assigns.Referencing parent assignsWhen a LiveView is mounted in a disconnected state, the Plug.Conn assigns will be available for reference via assign_new/3, allowing assigns to be shared for the initial HTTP request. The Plug.Conn assigns will not be available during the connected mount. Likewise, nested LiveView children have access to their parent's assigns on mount using assign_new, which allows assigns to be shared down the nested LiveView tree.Examples# controller conn |&gt; assign(:current_user, user) |&gt; LiveView.Controller.live_render(MyLive, session: %{&quot;user_id&quot; =&gt; user.id}) # LiveView mount def mount(_params, %{&quot;user_id&quot; =&gt; user_id}, socket) do {:ok, assign_new(socket, :current_user, fn -&gt; Accounts.get_user!(user_id) end)} end","ref":"Phoenix.LiveView.html#assign_new/3","title":"Phoenix.LiveView.assign_new/3","type":"function"},{"doc":"Cancels an upload for the given entry.Examples&lt;%= for entry &lt;- @uploads.avatar.entries do %&gt; ... &lt;button phx-click=&quot;cancel-upload&quot; phx-value-ref=&quot;&lt;%= entry.ref %&gt;&quot;&gt;cancel&lt;/button&gt; &lt;% end %&gt; def handle_event(&quot;cancel-upload&quot;, %{&quot;ref&quot; =&gt; ref}, socket) do {:noreply, cancel_upload(socket, :avatar, ref)} end","ref":"Phoenix.LiveView.html#cancel_upload/3","title":"Phoenix.LiveView.cancel_upload/3","type":"function"},{"doc":"Clears the flash.Examplesiex&gt; clear_flash(socket)","ref":"Phoenix.LiveView.html#clear_flash/1","title":"Phoenix.LiveView.clear_flash/1","type":"function"},{"doc":"Clears a key from the flash.Examplesiex&gt; clear_flash(socket, :info)","ref":"Phoenix.LiveView.html#clear_flash/2","title":"Phoenix.LiveView.clear_flash/2","type":"function"},{"doc":"Returns true if the socket is connected.Useful for checking the connectivity status when mounting the view. For example, on initial page render, the view is mounted statically, rendered, and the HTML is sent to the client. Once the client connects to the server, a LiveView is then spawned and mounted statefully within a process. Use connected?/1 to conditionally perform stateful work, such as subscribing to pubsub topics, sending messages, etc.Examplesdefmodule DemoWeb.ClockLive do use Phoenix.LiveView ... def mount(_params, _session, socket) do if connected?(socket), do: :timer.send_interval(1000, self(), :tick) {:ok, assign(socket, date: :calendar.local_time())} end def handle_info(:tick, socket) do {:noreply, assign(socket, date: :calendar.local_time())} end end","ref":"Phoenix.LiveView.html#connected?/1","title":"Phoenix.LiveView.connected?/1","type":"function"},{"doc":"Consumes the uploaded entries.Raises when there are still entries in progress. Typically called when submitting a form to handle the uploaded entries alongside the form data. For form submissions, it is guaranteed that all entries have completed before the submit event is invoked. Once entries are consumed, they are removed from the upload.Examplesdef handle_event(&quot;save&quot;, _params, socket) do uploaded_files = consume_uploaded_entries(socket, :avatar, fn %{path: path}, _entry -&gt; dest = Path.join(&quot;priv/static/uploads&quot;, Path.basename(path)) File.cp!(path, dest) Routes.static_path(socket, &quot;/uploads/\#{Path.basename(dest)}&quot;) end) {:noreply, update(socket, :uploaded_files, &amp;(&amp;1 ++ uploaded_files))} end","ref":"Phoenix.LiveView.html#consume_uploaded_entries/3","title":"Phoenix.LiveView.consume_uploaded_entries/3","type":"function"},{"doc":"Consumes an individual uploaded entry.Raises when the entry is still in progress. Typically called when submitting a form to handle the uploaded entries alongside the form data. Once entries are consumed, they are removed from the upload.This is a lower-level feature than consume_uploaded_entries/3 and useful for scenarios where you want to consume entries as they are individually completed.Examplesdef handle_event(&quot;save&quot;, _params, socket) do case uploaded_entries(socket, :avatar) do {[_|_] = entries, []} -&gt; uploaded_files = for entry &lt;- entries do consume_uploaded_entry(socket, entry, fn %{path: path} -&gt; dest = Path.join(&quot;priv/static/uploads&quot;, Path.basename(path)) File.cp!(path, dest) Routes.static_path(socket, &quot;/uploads/\#{Path.basename(dest)}&quot;) end) end {:noreply, update(socket, :uploaded_files, &amp;(&amp;1 ++ uploaded_files))} _ -&gt; {:noreply, socket} end end","ref":"Phoenix.LiveView.html#consume_uploaded_entry/3","title":"Phoenix.LiveView.consume_uploaded_entry/3","type":"function"},{"doc":"Revokes a previously allowed upload from allow_upload/3.Examplesdisallow_upload(socket, :avatar)","ref":"Phoenix.LiveView.html#disallow_upload/2","title":"Phoenix.LiveView.disallow_upload/2","type":"function"},{"doc":"Accesses the connect info from the socket to use on connected mount.Connect info are only sent when the client connects to the server and only remain available during mount. nil is returned when called in a disconnected state and a RuntimeError is raised if called after mount.ExamplesFirst, when invoking the LiveView socket, you need to declare the connect_info you want to receive. Typically, it includes at least the session but it may include other keys, such as :peer_data. See Phoenix.Endpoint.socket/3:socket &quot;/live&quot;, Phoenix.LiveView.Socket, websocket: [connect_info: [:peer_data, session: @session_options]]Those values can now be accessed on the connected mount as get_connect_info/1:def mount(_params, _session, socket) do if info = get_connect_info(socket) do {:ok, assign(socket, ip: info.peer_data.address)} else {:ok, assign(socket, ip: nil)} end end","ref":"Phoenix.LiveView.html#get_connect_info/1","title":"Phoenix.LiveView.get_connect_info/1","type":"function"},{"doc":"Accesses the connect params sent by the client for use on connected mount.Connect params are only sent when the client connects to the server and only remain available during mount. nil is returned when called in a disconnected state and a RuntimeError is raised if called after mount.Reserved paramsThe following params have special meaning in LiveView:&quot;_csrf_token&quot; - the CSRF Token which must be explicitly set by the user when connecting&quot;_mounts&quot; - the number of times the current LiveView is mounted. It is 0 on first mount, then increases on each reconnect. It resets when navigating away from the current LiveView or on errors&quot;_track_static&quot; - set automatically with a list of all href/src from tags with the &quot;phx-track-static&quot; annotation in them. If there are no such tags, nothing is sentExamplesdef mount(_params, _session, socket) do {:ok, assign(socket, width: get_connect_params(socket)[&quot;width&quot;] || @width)} end","ref":"Phoenix.LiveView.html#get_connect_params/1","title":"Phoenix.LiveView.get_connect_params/1","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_call/3","title":"Phoenix.LiveView.handle_call/3","type":"callback"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_cast/2","title":"Phoenix.LiveView.handle_cast/2","type":"callback"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_event/3","title":"Phoenix.LiveView.handle_event/3","type":"callback"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_info/2","title":"Phoenix.LiveView.handle_info/2","type":"callback"},{"doc":"","ref":"Phoenix.LiveView.html#c:handle_params/3","title":"Phoenix.LiveView.handle_params/3","type":"callback"},{"doc":"The LiveView entry-point.For each LiveView in the root of a template, mount/3 is invoked twice: once to do the initial page load and again to establish the live socket.It expects three parameters:params - a map of string keys which contain public information that can be set by the user. The map contains the query params as well as any router path parameter. If the LiveView was not mounted at the router, this argument is the atom :not_mounted_at_routersession - the connection sessionsocket - the LiveView socketIt must return either {:ok, socket} or {:ok, socket, options}, where options is one of::temporary_assigns - a keyword list of assigns that are temporary and must be reset to their value after every render. Note that once the value is reset, it won't be re-rendered again until it is explicitly assigned:layout - the optional layout to be used by the LiveView","ref":"Phoenix.LiveView.html#c:mount/3","title":"Phoenix.LiveView.mount/3","type":"callback"},{"doc":"Pushes an event to the client to be consumed by hooks.Note: events will be dispatched to all active hooks on the client who are handling the given event. Scoped events can be achieved by namespacing your event names.Examples{:noreply, push_event(socket, &quot;scores&quot;, %{points: 100, user: &quot;josé&quot;})}","ref":"Phoenix.LiveView.html#push_event/3","title":"Phoenix.LiveView.push_event/3","type":"function"},{"doc":"Annotates the socket for navigation within the current LiveView.When navigating to the current LiveView, handle_params/3 is immediately invoked to handle the change of params and URL state. Then the new state is pushed to the client, without reloading the whole page while also maintaining the current scroll position. For live redirects to another LiveView, use push_redirect/2.Options:to - the required path to link to. It must always be a local path:replace - the flag to replace the current history or push a new state. Defaults false.Examples{:noreply, push_patch(socket, to: &quot;/&quot;)} {:noreply, push_patch(socket, to: &quot;/&quot;, replace: true)}","ref":"Phoenix.LiveView.html#push_patch/2","title":"Phoenix.LiveView.push_patch/2","type":"function"},{"doc":"Annotates the socket for navigation to another LiveView.The current LiveView will be shutdown and a new one will be mounted in its place, without reloading the whole page. This can also be used to remount the same LiveView, in case you want to start fresh. If you want to navigate to the same LiveView without remounting it, use push_patch/2 instead.Options:to - the required path to link to. It must always be a local path:replace - the flag to replace the current history or push a new state. Defaults false.Examples{:noreply, push_redirect(socket, to: &quot;/&quot;)} {:noreply, push_redirect(socket, to: &quot;/&quot;, replace: true)}","ref":"Phoenix.LiveView.html#push_redirect/2","title":"Phoenix.LiveView.push_redirect/2","type":"function"},{"doc":"Adds a flash message to the socket to be displayed.Note: While you can use put_flash/3 inside a Phoenix.LiveComponent, components have their own @flash assigns. The @flash assign in a component is only copied to its parent LiveView if the component calls push_redirect/2 or push_patch/2.Note: You must also place the Phoenix.LiveView.Router.fetch_live_flash/2 plug in your browser's pipeline in place of fetch_flash to be supported, for example:import Phoenix.LiveView.Router pipeline :browser do ... plug :fetch_live_flash endExamplesiex&gt; put_flash(socket, :info, &quot;It worked!&quot;) iex&gt; put_flash(socket, :error, &quot;You can&#39;t access that page&quot;)","ref":"Phoenix.LiveView.html#put_flash/3","title":"Phoenix.LiveView.put_flash/3","type":"function"},{"doc":"Annotates the socket for redirect to a destination path.Note: LiveView redirects rely on instructing client to perform a window.location update on the provided redirect location. The whole page will be reloaded and all state will be discarded.Options:to - the path to redirect to. It must always be a local path:external - an external path to redirect to","ref":"Phoenix.LiveView.html#redirect/2","title":"Phoenix.LiveView.redirect/2","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#c:render/1","title":"Phoenix.LiveView.render/1","type":"callback"},{"doc":"Asynchronously updates a Phoenix.LiveComponent with new assigns.The component that is updated must be stateful (the :id in the assigns must match the :id associated with the component) and the component must be mounted within the current LiveView.If this call is executed from a process which is not a LiveView nor a LiveComponent, the pid parameter has to be specified.When the component receives the update, the optional preload/1 callback is invoked, then the updated values are merged with the component's assigns and update/2 is called for the updated component(s).While a component may always be updated from the parent by updating some parent assigns which will re-render the child, thus invoking update/3 on the child component, send_update/3 is useful for updating a component that entirely manages its own state, as well as messaging between components mounted in the same LiveView.Note: send_update/3 cannot update a LiveComponent that is mounted in a different LiveView. To update a component in a different LiveView you must send a message to the LiveView process that the LiveComponent is mounted within (often via Phoenix.PubSub).Examplesdef handle_event(&quot;cancel-order&quot;, _, socket) do ... send_update(Cart, id: &quot;cart&quot;, status: &quot;cancelled&quot;) {:noreply, socket} end def handle_event(&quot;cancel-order-asynchronously&quot;, _, socket) do ... pid = self() Task.async(fn -&gt; # Do domething asynchronously send_update(pid, Cart, id: &quot;cart&quot;, status: &quot;cancelled&quot;) end) {:noreply, socket} end","ref":"Phoenix.LiveView.html#send_update/3","title":"Phoenix.LiveView.send_update/3","type":"function"},{"doc":"Similar to send_update/3 but the update will be delayed according to the given time_in_milliseconds.Examplesdef handle_event(&quot;cancel-order&quot;, _, socket) do ... send_update_after(Cart, [id: &quot;cart&quot;, status: &quot;cancelled&quot;], 3000) {:noreply, socket} end def handle_event(&quot;cancel-order-asynchronously&quot;, _, socket) do ... pid = self() Task.async(fn -&gt; # Do domething asynchronously send_update_after(pid, Cart, [id: &quot;cart&quot;, status: &quot;cancelled&quot;], 3000) end) {:noreply, socket} end","ref":"Phoenix.LiveView.html#send_update_after/4","title":"Phoenix.LiveView.send_update_after/4","type":"function"},{"doc":"Returns true if the socket is connected and the tracked static assets have changed.This function is useful to detect if the client is running on an outdated version of the marked static files. It works by comparing the static paths sent by the client with the one on the server.Note: this functionality requires Phoenix v1.5.2 or later.To use this functionality, the first step is to annotate which static files you want to be tracked by LiveView, with the phx-track-static. For example:&lt;link phx-track-static rel=&quot;stylesheet&quot; href=&quot;&lt;%= Routes.static_path(@conn, &quot;/css/app.css&quot;) %&gt;&quot;/&gt; &lt;script defer phx-track-static type=&quot;text/javascript&quot; src=&quot;&lt;%= Routes.static_path(@conn, &quot;/js/app.js&quot;) %&gt;&quot;&gt;&lt;/script&gt;Now, whenever LiveView connects to the server, it will send a copy src or href attributes of all tracked statics and compare those values with the latest entries computed by mix phx.digest in the server.The tracked statics on the client will match the ones on the server the huge majority of times. However, if there is a new deployment, those values may differ. You can use this function to detect those cases and show a banner to the user, asking them to reload the page. To do so, first set the assign on mount:def mount(params, session, socket) do {:ok, assign(socket, static_changed?: static_changed?(socket))} endAnd then in your views:&lt;%= if @static_changed? do %&gt; &lt;div id=&quot;reload-static&quot;&gt; The app has been updated. Click here to &lt;a href=&quot;#&quot; onclick=&quot;window.location.reload()&quot;&gt;reload&lt;/a&gt;. &lt;/div&gt; &lt;% end %&gt;If you prefer, you can also send a JavaScript script that immediately reloads the page.Note: only set phx-track-static on your own assets. For example, do not set it in external JavaScript files:&lt;script defer phx-track-static type=&quot;text/javascript&quot; src=&quot;https://ajax.googleapis.com/ajax/libs/jquery/3.4.1/jquery.min.js&quot;&gt;&lt;/script&gt;Because you don't actually serve the file above, LiveView will interpret the static above as missing, and this function will return true.","ref":"Phoenix.LiveView.html#static_changed?/1","title":"Phoenix.LiveView.static_changed?/1","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#c:terminate/2","title":"Phoenix.LiveView.terminate/2","type":"callback"},{"doc":"Returns the transport pid of the socket.Raises ArgumentError if the socket is not connected.Examplesiex&gt; transport_pid(socket) #PID&lt;0.107.0&gt;","ref":"Phoenix.LiveView.html#transport_pid/1","title":"Phoenix.LiveView.transport_pid/1","type":"function"},{"doc":"Updates an existing key in the socket assigns.The update function receives the current key's value and returns the updated value. Raises if the key does not exist.Examplesiex&gt; update(socket, :count, fn count -&gt; count + 1 end) iex&gt; update(socket, :count, &amp;(&amp;1 + 1))","ref":"Phoenix.LiveView.html#update/3","title":"Phoenix.LiveView.update/3","type":"function"},{"doc":"Returns the completed and in progress entries for the upload.Examplescase uploaded_entries(socket, :photos) do {[_ | _] = completed, []} -&gt; # all entries are completed {[], [_ | _] = in_progress} -&gt; # all entries are still in progress end","ref":"Phoenix.LiveView.html#uploaded_entries/2","title":"Phoenix.LiveView.uploaded_entries/2","type":"function"},{"doc":"","ref":"Phoenix.LiveView.html#t:unsigned_params/0","title":"Phoenix.LiveView.unsigned_params/0","type":"type"},{"doc":"The Controller for LiveView rendering.","ref":"Phoenix.LiveView.Controller.html","title":"Phoenix.LiveView.Controller","type":"module"},{"doc":"Renders a live view from a Plug request and sends an HTML response.Before rendering, the @live_module assign will be added to the connection assigns for reference.OptionsSee Phoenix.LiveView.Helpers.live_render/3 for all supported options.Examplesdefmodule ThermostatController do ... import Phoenix.LiveView.Controller def show(conn, %{&quot;id&quot; =&gt; thermostat_id}) do live_render(conn, ThermostatLive, session: %{ &quot;thermostat_id&quot; =&gt; id, &quot;current_user_id&quot; =&gt; get_session(conn, :user_id) }) end end","ref":"Phoenix.LiveView.Controller.html#live_render/3","title":"Phoenix.LiveView.Controller.live_render/3","type":"function"},{"doc":"A collection of helpers to be imported into your views.","ref":"Phoenix.LiveView.Helpers.html","title":"Phoenix.LiveView.Helpers","type":"module"},{"doc":"Renders a Phoenix.LiveComponent within a parent LiveView.While LiveViews can be nested, each LiveView starts its own process. A LiveComponent provides similar functionality to LiveView, except they run in the same process as the LiveView, with its own encapsulated state.LiveComponent comes in two shapes, stateful and stateless. See Phoenix.LiveComponent for more information.ExamplesAll of the assigns given are forwarded directly to the live_component:&lt;%= live_component(@socket, MyApp.WeatherComponent, id: &quot;thermostat&quot;, city: &quot;Kraków&quot;) %&gt;Note the :id won't necessarily be used as the DOM ID. That's up to the component. However, note that the :id has a special meaning: whenever an :id is given, the component becomes stateful. Otherwise, :id is always set to nil.","ref":"Phoenix.LiveView.Helpers.html#live_component/4","title":"Phoenix.LiveView.Helpers.live_component/4","type":"macro"},{"doc":"Builds a file input tag for a LiveView upload.Options may be passed through to the tag builder for custom attributes.Drag and DropDrag and drop is supported by annotating the droppable container with a phx-drop-target attribute pointing to the DOM ID of the file input. By default, the file input ID is the upload ref, so the following markup is all that is required for drag and drop support:&lt;div class=&quot;container&quot; phx-drop-target=&quot;&lt;%= @uploads.avatar.ref %&gt;&quot;&gt; ... &lt;%= live_file_input @uploads.avatar %&gt; &lt;/div&gt;Examples&lt;%= live_file_input @uploads.avatar %&gt;","ref":"Phoenix.LiveView.Helpers.html#live_file_input/2","title":"Phoenix.LiveView.Helpers.live_file_input/2","type":"function"},{"doc":"Returns the flash message from the LiveView flash assign.Examples&lt;p class=&quot;alert alert-info&quot;&gt;&lt;%= live_flash(@flash, :info) %&gt;&lt;/p&gt; &lt;p class=&quot;alert alert-danger&quot;&gt;&lt;%= live_flash(@flash, :error) %&gt;&lt;/p&gt;","ref":"Phoenix.LiveView.Helpers.html#live_flash/2","title":"Phoenix.LiveView.Helpers.live_flash/2","type":"function"},{"doc":"Generates an image preview on the client for a selected file.Examples&lt;%= for entry &lt;- @uploads.avatar.entries do %&gt; &lt;%= live_img_preview entry, width: 75 %&gt; &lt;% end %&gt;","ref":"Phoenix.LiveView.Helpers.html#live_img_preview/2","title":"Phoenix.LiveView.Helpers.live_img_preview/2","type":"function"},{"doc":"Generates a link that will patch the current LiveView.When navigating to the current LiveView, Phoenix.LiveView.handle_params/3 is immediately invoked to handle the change of params and URL state. Then the new state is pushed to the client, without reloading the whole page while also maintaining the current scroll position. For live redirects to another LiveView, use live_redirect/2.Options:to - the required path to link to.:replace - the flag to replace the current history or push a new state. Defaults false.All other options are forwarded to the anchor tag.Examples&lt;%= live_patch &quot;home&quot;, to: Routes.page_path(@socket, :index) %&gt; &lt;%= live_patch &quot;next&quot;, to: Routes.live_path(@socket, MyLive, @page + 1) %&gt; &lt;%= live_patch to: Routes.live_path(@socket, MyLive, dir: :asc), replace: false do %&gt; Sort By Price &lt;% end %&gt;","ref":"Phoenix.LiveView.Helpers.html#live_patch/2","title":"Phoenix.LiveView.Helpers.live_patch/2","type":"function"},{"doc":"Generates a link that will redirect to a new LiveView.The current LiveView will be shut down and a new one will be mounted in its place, without reloading the whole page. This can also be used to remount the same LiveView, in case you want to start fresh. If you want to navigate to the same LiveView without remounting it, use live_patch/2 instead.Options:to - the required path to link to.:replace - the flag to replace the current history or push a new state. Defaults false.All other options are forwarded to the anchor tag.Examples&lt;%= live_redirect &quot;home&quot;, to: Routes.page_path(@socket, :index) %&gt; &lt;%= live_redirect &quot;next&quot;, to: Routes.live_path(@socket, MyLive, @page + 1) %&gt; &lt;%= live_redirect to: Routes.live_path(@socket, MyLive, dir: :asc), replace: false do %&gt; Sort By Price &lt;% end %&gt;","ref":"Phoenix.LiveView.Helpers.html#live_redirect/2","title":"Phoenix.LiveView.Helpers.live_redirect/2","type":"function"},{"doc":"Renders a LiveView within an originating plug request or within a parent LiveView.Options:session - the map of extra session data to be serialized and sent to the client. Note that all session data currently in the connection is automatically available in LiveViews. You can use this option to provide extra data. Also note that the keys in the session are strings keys, as a reminder that data has to be serialized first.:container - an optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;}. By default it uses the module definition container. See the &quot;Containers&quot; section below for more information.:id - both the DOM ID and the ID to uniquely identify a LiveView. An :id is automatically generated when rendering root LiveViews but it is a required option when rendering a child LiveView.:router - an optional router that enables this LiveView to perform live navigation. Only a single LiveView in a page may have the :router set. LiveViews defined at the router with the live macro automatically have the :router option set.Examples# within eex template &lt;%= live_render(@conn, MyApp.ThermostatLive) %&gt; # within leex template &lt;%= live_render(@socket, MyApp.ThermostatLive, id: &quot;thermostat&quot;) %&gt;ContainersWhen a LiveView is rendered, its contents are wrapped in a container. By default, said container is a div tag with a handful of LiveView specific attributes.The container can be customized in different ways:You can change the default container on use Phoenix.LiveView:use Phoenix.LiveView, container: {:tr, id: &quot;foo-bar&quot;}You can override the container tag and pass extra attributes when calling live_render (as well as on your live call in your router):live_render socket, MyLiveView, container: {:tr, class: &quot;highlight&quot;}","ref":"Phoenix.LiveView.Helpers.html#live_render/3","title":"Phoenix.LiveView.Helpers.live_render/3","type":"function"},{"doc":"Renders a title tag with automatic prefix/suffix on @page_title updates.Examples&lt;%= live_title_tag assigns[:page_title] || &quot;Welcome&quot;, prefix: &quot;MyApp – &quot; %&gt; &lt;%= live_title_tag assigns[:page_title] || &quot;Welcome&quot;, suffix: &quot; – MyApp&quot; %&gt;","ref":"Phoenix.LiveView.Helpers.html#live_title_tag/2","title":"Phoenix.LiveView.Helpers.live_title_tag/2","type":"function"},{"doc":"Renders the @inner_block assign of a component with the given argument.&lt;%= render_block(@inner_block, value: @value)","ref":"Phoenix.LiveView.Helpers.html#render_block/2","title":"Phoenix.LiveView.Helpers.render_block/2","type":"macro"},{"doc":"Provides ~L sigil with HTML safe Live EEx syntax inside source files.iex&gt; ~L&quot;&quot;&quot; ...&gt; Hello &lt;%= &quot;world&quot; %&gt; ...&gt; &quot;&quot;&quot; {:safe, [&quot;Hello &quot;, &quot;world&quot;, &quot;\\n&quot;]}","ref":"Phoenix.LiveView.Helpers.html#sigil_L/2","title":"Phoenix.LiveView.Helpers.sigil_L/2","type":"macro"},{"doc":"Returns the entry errors for an upload.The following errors may be returned::too_large - The entry exceeds the :max_file_size constraint:too_many_files - The number of selected files exceeds the :max_entries constraint:not_accepted - The entry does not match the :accept MIME typesExamplesdef error_to_string(:too_large), do: &quot;Too large&quot; def error_to_string(:too_many_files), do: &quot;You have selected too many files&quot; def error_to_string(:not_accepted), do: &quot;You have selected an unacceptable file type&quot; &lt;%= for entry &lt;- @uploads.avatar.entries do %&gt; &lt;%= for err &lt;- upload_errors(@uploads.avatar, entry) do %&gt; &lt;div class=&quot;alert alert-danger&quot;&gt; &lt;%= error_to_string(err) %&gt; &lt;/div&gt; &lt;% end %&gt; &lt;% end %&gt;","ref":"Phoenix.LiveView.Helpers.html#upload_errors/2","title":"Phoenix.LiveView.Helpers.upload_errors/2","type":"function"},{"doc":"Provides LiveView routing for Phoenix routers.","ref":"Phoenix.LiveView.Router.html","title":"Phoenix.LiveView.Router","type":"module"},{"doc":"Fetches the LiveView and merges with the controller flash.Replaces the default :fetch_flash plug used by Phoenix.Router.Examplesdefmodule AppWeb.Router do use LiveGenWeb, :router import Phoenix.LiveView.Router pipeline :browser do ... plug :fetch_live_flash end ... end","ref":"Phoenix.LiveView.Router.html#fetch_live_flash/2","title":"Phoenix.LiveView.Router.fetch_live_flash/2","type":"function"},{"doc":"Defines a LiveView route.A LiveView can be routed to by using the live macro with a path and the name of the LiveView:live &quot;/thermostat&quot;, ThermostatLiveBy default, you can generate a route to this LiveView by using the live_path helper:live_path(@socket, ThermostatLive)Actions and live navigationIt is common for a LiveView to have multiple states and multiple URLs. For example, you can have a single LiveView that lists all articles on your web app. For each article there is an &quot;Edit&quot; button which, when pressed, opens up a modal on the same page to edit the article. It is a best practice to use live navigation in those cases, so when you click edit, the URL changes to &quot;/articles/1/edit&quot;, even though you are still within the same LiveView. Similarly, you may also want to show a &quot;New&quot; button, which opens up the modal to create new entries, and you want this to be reflected in the URL as &quot;/articles/new&quot;.In order to make it easier to recognize the current &quot;action&quot; your LiveView is on, you can pass the action option when defining LiveViews too:live &quot;/articles&quot;, ArticleLive.Index, :index live &quot;/articles/new&quot;, ArticleLive.Index, :new live &quot;/articles/:id/edit&quot;, ArticleLive.Index, :editWhen an action is given, the generated route helpers are named after the LiveView itself (in the same way as for a controller). For the example above, we will have:article_index_path(@socket, :index) article_index_path(@socket, :new) article_index_path(@socket, :edit, 123)The current action will always be available inside the LiveView as the @live_action assign, that can be used to render a LiveComponent:&lt;%= if @live_action == :new do %&gt; &lt;%= live_component @socket, MyAppWeb.ArticleLive.FormComponent %&gt; &lt;% end %&gt;Or can be used to show or hide parts of the template:&lt;%= if @live_action == :edit do %&gt; &lt;%= render(&quot;form.html&quot;, user: @user) %&gt; &lt;% end %&gt;Note that @live_action will be nil if no action is given on the route definition.Options:session - a map to be merged into the session, for example: %{&quot;my_key&quot; =&gt; 123}. The map keys must be strings.Can also be a &quot;MFA&quot; (module, function, arguments) tuple. That function will receive the connection and should return a map (with string keys) to be merged into the session. For example, {MyModule, :my_function, []} means MyModule.my_function(conn) is called.:layout - an optional tuple to specify the rendering layout for the LiveView. If set, this option will replace the current root layout.:container - an optional tuple for the HTML tag and DOM attributes to be used for the LiveView container. For example: {:li, style: &quot;color: blue;&quot;}. See Phoenix.LiveView.Helpers.live_render/3 for more information and examples.:as - optionally configures the named helper. Defaults to :live when using a LiveView without actions or defaults to the LiveView name when using actions.:metadata - a map to optional feed metadata used on telemetry events and route info, for example: %{route_name: :foo, access: :user}.Examplesdefmodule MyApp.Router use Phoenix.Router import Phoenix.LiveView.Router scope &quot;/&quot;, MyApp do pipe_through [:browser] live &quot;/thermostat&quot;, ThermostatLive live &quot;/clock&quot;, ClockLive live &quot;/dashboard&quot;, DashboardLive, layout: {MyApp.AlternativeView, &quot;app.html&quot;} end end iex&gt; MyApp.Router.Helpers.live_path(MyApp.Endpoint, MyApp.ThermostatLive) &quot;/thermostat&quot;","ref":"Phoenix.LiveView.Router.html#live/4","title":"Phoenix.LiveView.Router.live/4","type":"macro"},{"doc":"The LiveView socket for Phoenix Endpoints.This is typically mounted directly in your endpoint.socket &quot;/live&quot;, Phoenix.LiveView.Socket","ref":"Phoenix.LiveView.Socket.html","title":"Phoenix.LiveView.Socket","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Socket.html#t:assigns/0","title":"Phoenix.LiveView.Socket.assigns/0","type":"type"},{"doc":"","ref":"Phoenix.LiveView.Socket.html#t:fingerprints/0","title":"Phoenix.LiveView.Socket.fingerprints/0","type":"type"},{"doc":"","ref":"Phoenix.LiveView.Socket.html#t:t/0","title":"Phoenix.LiveView.Socket.t/0","type":"type"},{"doc":"Conveniences for testing Phoenix LiveViews.In LiveView tests, we interact with views via process communication in substitution of a browser. Like a browser, our test process receives messages about the rendered updates from the view which can be asserted against to test the life-cycle and behavior of LiveViews and their children.LiveView TestingThe life-cycle of a LiveView as outlined in the Phoenix.LiveView docs details how a view starts as a stateless HTML render in a disconnected socket state. Once the browser receives the HTML, it connects to the server and a new LiveView process is started, remounted in a connected socket state, and the view continues statefully. The LiveView test functions support testing both disconnected and connected mounts separately, for example:import Plug.Conn import Phoenix.ConnTest import Phoenix.LiveViewTest @endpoint MyEndpoint test &quot;disconnected and connected mount&quot;, %{conn: conn} do conn = get(conn, &quot;/my-path&quot;) assert html_response(conn, 200) =~ &quot;&lt;h1&gt;My Disconnected View&lt;/h1&gt;&quot; {:ok, view, html} = live(conn) end test &quot;redirected mount&quot;, %{conn: conn} do assert {:error, {:redirect, %{to: &quot;/somewhere&quot;}}} = live(conn, &quot;my-path&quot;) endHere, we start by using the familiar Phoenix.ConnTest function, get/2 to test the regular HTTP GET request which invokes mount with a disconnected socket. Next, live/1 is called with our sent connection to mount the view in a connected state, which starts our stateful LiveView process.In general, it's often more convenient to test the mounting of a view in a single step, provided you don't need the result of the stateless HTTP render. This is done with a single call to live/2, which performs the get step for us:test &quot;connected mount&quot;, %{conn: conn} do {:ok, _view, html} = live(conn, &quot;/my-path&quot;) assert html =~ &quot;&lt;h1&gt;My Connected View&lt;/h1&gt;&quot; endTesting EventsThe browser can send a variety of events to a LiveView via phx- bindings, which are sent to the handle_event/3 callback. To test events sent by the browser and assert on the rendered side effect of the event, use the render_* functions:render_click/1 - sends a phx-click event and value, returning the rendered result of the handle_event/3 callback.render_focus/2 - sends a phx-focus event and value, returning the rendered result of the handle_event/3 callback.render_blur/1 - sends a phx-blur event and value, returning the rendered result of the handle_event/3 callback.render_submit/1 - sends a form phx-submit event and value, returning the rendered result of the handle_event/3 callback.render_change/1 - sends a form phx-change event and value, returning the rendered result of the handle_event/3 callback.render_keydown/1 - sends a form phx-keydown event and value, returning the rendered result of the handle_event/3 callback.render_keyup/1 - sends a form phx-keyup event and value, returning the rendered result of the handle_event/3 callback.render_hook/3 - sends a hook event and value, returning the rendered result of the handle_event/3 callback.For example:{:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert render_click(view, :inc) =~ &quot;The temperature is: 31℉&quot; assert render_click(view, :set_temp, 35) =~ &quot;The temperature is: 35℉&quot; assert render_submit(view, :save, %{deg: 30}) =~ &quot;The temperature is: 30℉&quot; assert render_change(view, :validate, %{deg: -30}) =~ &quot;invalid temperature&quot; assert render_keydown(view, :key, :ArrowUp) =~ &quot;The temperature is: 31℉&quot; assert render_keydown(view, :key, :ArrowDown) =~ &quot;The temperature is: 30℉&quot;Testing regular messagesLiveViews are GenServer's under the hood, and can send and receive messages just like any other server. To test the side effects of sending or receiving messages, simply message the view and use the render function to test the result:send(view.pid, {:set_temp, 50}) assert render(view) =~ &quot;The temperature is: 50℉&quot;Testing componentsThere are two main mechanisms for testing components. To test stateless components or just a regular rendering of a component, one can use render_component/2:assert render_component(MyComponent, id: 123, user: %User{}) =~ &quot;some markup in component&quot;If you want to test how components are mounted by a LiveView and interact with DOM events, you can use the regular live/2 macro to build the LiveView with the component and then scope events by passing the view and a DOM selector in a list:{:ok, view, html} = live(conn, &quot;/users&quot;) html = view |&gt; element(&quot;#user-13 a&quot;, &quot;Delete&quot;) |&gt; render_click() refute html =~ &quot;user-13&quot; refute view |&gt; element(&quot;#user-13&quot;) |&gt; has_element?()In the example above, LiveView will lookup for an element with ID=user-13 and retrieve its phx-target. If phx-target points to a component, that will be the component used, otherwise it will fallback to the view.","ref":"Phoenix.LiveViewTest.html","title":"Phoenix.LiveViewTest","type":"module"},{"doc":"Asserts a live patch will happen within timeout.It always returns :ok. To assert on the flash message, you can assert on the result of the rendered LiveView.Examplesrender_click(view, :event_that_triggers_patch) assert_patch view, &quot;/path&quot;","ref":"Phoenix.LiveViewTest.html#assert_patch/3","title":"Phoenix.LiveViewTest.assert_patch/3","type":"function"},{"doc":"Asserts a live patch was performed.It always returns :ok. To assert on the flash message, you can assert on the result of the rendered LiveView.Examplesrender_click(view, :event_that_triggers_redirect) assert_patched view, &quot;/path&quot;","ref":"Phoenix.LiveViewTest.html#assert_patched/2","title":"Phoenix.LiveViewTest.assert_patched/2","type":"function"},{"doc":"Asserts an event will be pushed within timeout.Examplesassert_push_event view, &quot;scores&quot;, %{points: 100, user: &quot;josé&quot;}","ref":"Phoenix.LiveViewTest.html#assert_push_event/4","title":"Phoenix.LiveViewTest.assert_push_event/4","type":"macro"},{"doc":"Asserts a redirect will happen within timeout.It returns the flash messages from said redirect, if any. Note the flash will contain string keys.Examplesrender_click(view, :event_that_triggers_redirect) flash = assert_redirect view, &quot;/path&quot; assert flash[&quot;info&quot;] == &quot;Welcome&quot;","ref":"Phoenix.LiveViewTest.html#assert_redirect/3","title":"Phoenix.LiveViewTest.assert_redirect/3","type":"function"},{"doc":"Asserts a redirect was performed.It returns the flash messages from said redirect, if any. Note the flash will contain string keys.Examplesrender_click(view, :event_that_triggers_redirect) flash = assert_redirected view, &quot;/path&quot; assert flash[&quot;info&quot;] == &quot;Welcome&quot;","ref":"Phoenix.LiveViewTest.html#assert_redirected/2","title":"Phoenix.LiveViewTest.assert_redirected/2","type":"function"},{"doc":"Asserts a hook reply was returned from a handle_event callback.Examplesassert_reply view, %{result: &quot;ok&quot;, transaction_id: _}","ref":"Phoenix.LiveViewTest.html#assert_reply/3","title":"Phoenix.LiveViewTest.assert_reply/3","type":"macro"},{"doc":"Returns an element to scope a function to.It expects the current LiveView, a query selector, and a text filter.An optional text filter may be given to filter the results by the query selector. If the text filter is a string or a regex, it will match any element that contains the string or matches the regex. After the text filter is applied, only one element must remain, otherwise an error is raised.If no text filter is given, then the query selector itself must return a single element.assert view |&gt; element(&quot;#term a:first-child()&quot;, &quot;Increment&quot;) |&gt; render() =~ &quot;Increment&lt;/a&gt;&quot;","ref":"Phoenix.LiveViewTest.html#element/3","title":"Phoenix.LiveViewTest.element/3","type":"function"},{"doc":"Builds a file input for testing uploads within a form.Given the form DOM selector, the upload name, and a list of maps of client metadata for the upload, the returned file input can be passed to render_upload/2.Client metadata takes the following form::last_modified - the last modified timestamp:name - the name of the file:content - the binary content of the file:size - the byte size of the content:type - the MIME type of the fileExamplesavatar = file_input(lv, &quot;#my-form-id&quot;, :avatar, [%{ last_modified: 1_594_171_879_000, name: &quot;myfile.jpeg&quot;, content: File.read!(&quot;myfile.jpg&quot;), size: 1_396_009, type: &quot;image/jpeg&quot; }]) assert render_upload(avatar, &quot;foo.jpeg&quot;) =~ &quot;100%&quot;","ref":"Phoenix.LiveViewTest.html#file_input/4","title":"Phoenix.LiveViewTest.file_input/4","type":"macro"},{"doc":"Gets the nested LiveView child by child_id from the parent LiveView.Examples{:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert clock_view = find_live_child(view, &quot;clock&quot;) assert render_click(clock_view, :snooze) =~ &quot;snoozing&quot;","ref":"Phoenix.LiveViewTest.html#find_live_child/2","title":"Phoenix.LiveViewTest.find_live_child/2","type":"function"},{"doc":"Follows the redirect from a render_* action.Imagine you have a LiveView that redirects on a render_click event. You can make it sure it immediately redirects after the render_click action by calling follow_redirect/3:live_view |&gt; render_click(&quot;redirect&quot;) |&gt; follow_redirect(conn)follow_redirect/3 expects a connection as second argument. This is the connection that will be used to perform the underlying request.If the LiveView redirects with a live redirect, this macro returns {:ok, live_view, disconnected_html} with the content of the new LiveView, the same as the live/3 macro. If the LiveView redirects with a regular redirect, this macro returns {:ok, conn} with the rendered redirected page. In any other case, this macro raises.Finally, note that you can optionally assert on the path you are being redirected to by passing a third argument:live_view |&gt; render_click(&quot;redirect&quot;) |&gt; follow_redirect(conn, &quot;/redirected/page&quot;)","ref":"Phoenix.LiveViewTest.html#follow_redirect/3","title":"Phoenix.LiveViewTest.follow_redirect/3","type":"macro"},{"doc":"Returns a form element to scope a function to.It expects the current LiveView, a query selector, and the form data. The query selector must return a single element.The form data will be validated directly against the form markup and make sure the data you are changing/submitting actually exists, failing otherwise.Examplesassert view |&gt; form(&quot;#term&quot;, user: %{name: &quot;hello&quot;}) |&gt; render_submit() =~ &quot;Name updated&quot;This function is meant to mimic what the user can actually do, so you cannot set hidden input values. However, hidden values can be given when calling render_submit/2 or render_change/2, see their docs for examples.","ref":"Phoenix.LiveViewTest.html#form/3","title":"Phoenix.LiveViewTest.form/3","type":"function"},{"doc":"Checks if the given element exists on the page.Examplesassert view |&gt; element(&quot;#some-element&quot;) |&gt; has_element?()","ref":"Phoenix.LiveViewTest.html#has_element?/1","title":"Phoenix.LiveViewTest.has_element?/1","type":"function"},{"doc":"Checks if the given selector with text_filter is on view.See element/3 for more information.Examplesassert has_element?(view, &quot;#some-element&quot;)","ref":"Phoenix.LiveViewTest.html#has_element?/3","title":"Phoenix.LiveViewTest.has_element?/3","type":"function"},{"doc":"Spawns a connected LiveView process.If a path is given, then a regular get(conn, path) is done and the page is upgraded to a LiveView. If no path is given, it assumes a previously rendered %Plug.Conn{} is given, which will be converted to a LiveView immediately.Examples{:ok, view, html} = live(conn, &quot;/path&quot;) assert view.module = MyLive assert html =~ &quot;the count is 3&quot; assert {:error, {:redirect, %{to: &quot;/somewhere&quot;}}} = live(conn, &quot;/path&quot;)","ref":"Phoenix.LiveViewTest.html#live/2","title":"Phoenix.LiveViewTest.live/2","type":"macro"},{"doc":"Returns the current list of LiveView children for the parent LiveView.Children are returned in the order they appear in the rendered HTML.Examples{:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert [clock_view] = live_children(view) assert render_click(clock_view, :snooze) =~ &quot;snoozing&quot;","ref":"Phoenix.LiveViewTest.html#live_children/1","title":"Phoenix.LiveViewTest.live_children/1","type":"function"},{"doc":"Spawns a connected LiveView process mounted in isolation as the sole rendered element.Useful for testing LiveViews that are not directly routable, such as those built as small components to be re-used in multiple parents. Testing routable LiveViews is still recommended whenever possible since features such as live navigation require routable LiveViews.Options:connect_params - the map of params available in connected mount. See Phoenix.LiveView.get_connect_params/1 for more information.:session - the session to be given to the LiveViewAll other options are forwarded to the LiveView for rendering. Refer to Phoenix.LiveView.Helpers.live_render/3 for a list of supported render options.Examples{:ok, view, html} = live_isolated(conn, AppWeb.ClockLive, session: %{&quot;tz&quot; =&gt; &quot;EST&quot;})","ref":"Phoenix.LiveViewTest.html#live_isolated/3","title":"Phoenix.LiveViewTest.live_isolated/3","type":"macro"},{"doc":"Open the default browser to display current HTML of view_or_element.Examplesview |&gt; element(&quot;#term a:first-child()&quot;, &quot;Increment&quot;) |&gt; open_browser() assert view |&gt; form(&quot;#term&quot;, user: %{name: &quot;hello&quot;}) |&gt; open_browser() |&gt; render_submit() =~ &quot;Name updated&quot;","ref":"Phoenix.LiveViewTest.html#open_browser/2","title":"Phoenix.LiveViewTest.open_browser/2","type":"function"},{"doc":"Returns the most recent title that was updated via a page_title assign.Examplesrender_click(view, :event_that_triggers_page_title_update) assert page_title(view) =~ &quot;my title&quot;","ref":"Phoenix.LiveViewTest.html#page_title/1","title":"Phoenix.LiveViewTest.page_title/1","type":"function"},{"doc":"Performs a preflight upload request.Useful for testing external uploaders to retrieve the :external entry metadata.Examplesavatar = file_input(lv, &quot;form&quot;, :avatar, %{name: ..., ...}) assert {:ok, %{ref: _ref, config: %{chunk_size: _}}} = preflight_upload(avatar)","ref":"Phoenix.LiveViewTest.html#preflight_upload/1","title":"Phoenix.LiveViewTest.preflight_upload/1","type":"function"},{"doc":"Puts connect info to be used on LiveView connections.See Phoenix.LiveView.get_connect_info/1.","ref":"Phoenix.LiveViewTest.html#put_connect_info/2","title":"Phoenix.LiveViewTest.put_connect_info/2","type":"function"},{"doc":"Puts connect params to be used on LiveView connections.See Phoenix.LiveView.get_connect_params/1.","ref":"Phoenix.LiveViewTest.html#put_connect_params/2","title":"Phoenix.LiveViewTest.put_connect_params/2","type":"function"},{"doc":"Returns the HTML string of the rendered view or element.If a view is provided, the entire LiveView is rendered. If an element is provided, only that element is rendered.Examples{:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert render(view) =~ ~s|&lt;button id=&quot;alarm&quot;&gt;Snooze&lt;/div&gt;| assert view |&gt; element(&quot;#alarm&quot;) |&gt; render() == &quot;Snooze&quot;","ref":"Phoenix.LiveViewTest.html#render/1","title":"Phoenix.LiveViewTest.render/1","type":"function"},{"doc":"Sends a blur event given by element and returns the rendered result.The element is created with element/3 and must point to a single element on the page with a phx-blur attribute in it. The event name given set on phx-blur is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert view |&gt; element(&quot;#inactive&quot;) |&gt; render_blur() =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_blur/2","title":"Phoenix.LiveViewTest.render_blur/2","type":"function"},{"doc":"Sends a blur event to the view and returns the rendered result.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_blur(view, :inactive) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_blur/3","title":"Phoenix.LiveViewTest.render_blur/3","type":"function"},{"doc":"Sends a form change event given by element and returns the rendered result.The element is created with element/3 and must point to a single element on the page with a phx-change attribute in it. The event name given set on phx-change is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values.If you need to pass any extra values or metadata, such as the &quot;_target&quot; parameter, you can do so by giving a map under the value argument.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert view |&gt; element(&quot;form&quot;) |&gt; render_change(%{deg: 123}) =~ &quot;123 exceeds limits&quot; # Passing metadata {:ok, view, html} = live(conn, &quot;/thermo&quot;) assert view |&gt; element(&quot;form&quot;) |&gt; render_change(%{_target: [&quot;deg&quot;], deg: 123}) =~ &quot;123 exceeds limits&quot;As with render_submit/2, hidden input field values can be provided like so:refute view |&gt; form(&quot;#term&quot;, user: %{name: &quot;hello&quot;}) |&gt; render_change(%{user: %{&quot;hidden_field&quot; =&gt; &quot;example&quot;}}) =~ &quot;can&#39;t be blank&quot;","ref":"Phoenix.LiveViewTest.html#render_change/2","title":"Phoenix.LiveViewTest.render_change/2","type":"function"},{"doc":"Sends a form change event to the view and returns the rendered result.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_change(view, :validate, %{deg: 123}) =~ &quot;123 exceeds limits&quot;","ref":"Phoenix.LiveViewTest.html#render_change/3","title":"Phoenix.LiveViewTest.render_change/3","type":"function"},{"doc":"Sends a click event given by element and returns the rendered result.The element is created with element/3 and must point to a single element on the page with a phx-click attribute in it. The event name given set on phx-click is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument.If the element is does not have a phx-click attribute but it is a link (the &lt;a&gt; tag), the link will be followed accordingly:if the link is a live_patch, the current view will be patchedif the link is a live_redirect, this function will return {:error, {:live_redirect, %{to: url}}}, which can be followed with follow_redirect/2if the link is a regular link, this function will return {:error, {:redirect, %{to: url}}}, which can be followed with follow_redirect/2It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert view |&gt; element(&quot;buttons&quot;, &quot;Increment&quot;) |&gt; render_click() =~ &quot;The temperature is: 30℉&quot;","ref":"Phoenix.LiveViewTest.html#render_click/2","title":"Phoenix.LiveViewTest.render_click/2","type":"function"},{"doc":"Sends a click event to the view with value and returns the rendered result.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temperature is: 30℉&quot; assert render_click(view, :inc) =~ &quot;The temperature is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_click/3","title":"Phoenix.LiveViewTest.render_click/3","type":"function"},{"doc":"Mounts, updates and renders a component.If the component uses the @myself assigns, then an id must be given to it is marked as stateful.Examplesassert render_component(MyComponent, id: 123, user: %User{}) =~ &quot;some markup in component&quot; assert render_component(MyComponent, %{id: 123, user: %User{}}, router: SomeRouter) =~ &quot;some markup in component&quot;","ref":"Phoenix.LiveViewTest.html#render_component/3","title":"Phoenix.LiveViewTest.render_component/3","type":"macro"},{"doc":"Sends a focus event given by element and returns the rendered result.The element is created with element/3 and must point to a single element on the page with a phx-focus attribute in it. The event name given set on phx-focus is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert view |&gt; element(&quot;#inactive&quot;) |&gt; render_focus() =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_focus/2","title":"Phoenix.LiveViewTest.render_focus/2","type":"function"},{"doc":"Sends a focus event to the view and returns the rendered result.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_focus(view, :inactive) =~ &quot;Tap to wake&quot;","ref":"Phoenix.LiveViewTest.html#render_focus/3","title":"Phoenix.LiveViewTest.render_focus/3","type":"function"},{"doc":"Sends a hook event to the view or an element and returns the rendered result.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_hook(view, :refresh, %{deg: 32}) =~ &quot;The temp is: 32℉&quot;If you are pushing events from a hook to a component, then you must pass an element, created with element/3, as first argument and it must point to a single element on the page with a phx-target attribute in it:{:ok, view, _html} = live(conn, &quot;/thermo&quot;) assert view |&gt; element(&quot;#thermo-component&quot;) |&gt; render_hook(:refresh, %{deg: 32}) =~ &quot;The temp is: 32℉&quot;","ref":"Phoenix.LiveViewTest.html#render_hook/3","title":"Phoenix.LiveViewTest.render_hook/3","type":"function"},{"doc":"Sends a keydown event given by element and returns the rendered result.The element is created with element/3 and must point to a single element on the page with a phx-keydown or phx-window-keydown attribute in it. The event name given set on phx-keydown is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert view |&gt; element(&quot;#inc&quot;) |&gt; render_keydown() =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keydown/2","title":"Phoenix.LiveViewTest.render_keydown/2","type":"function"},{"doc":"Sends a keydown event to the view and returns the rendered result.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_keydown(view, :inc) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keydown/3","title":"Phoenix.LiveViewTest.render_keydown/3","type":"function"},{"doc":"Sends a keyup event given by element and returns the rendered result.The element is created with element/3 and must point to a single element on the page with a phx-keyup or phx-window-keyup attribute in it. The event name given set on phx-keyup is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values can be given with the value argument.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert view |&gt; element(&quot;#inc&quot;) |&gt; render_keyup() =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keyup/2","title":"Phoenix.LiveViewTest.render_keyup/2","type":"function"},{"doc":"Sends a keyup event to the view and returns the rendered result.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_keyup(view, :inc) =~ &quot;The temp is: 31℉&quot;","ref":"Phoenix.LiveViewTest.html#render_keyup/3","title":"Phoenix.LiveViewTest.render_keyup/3","type":"function"},{"doc":"Simulates a live_patch to the given path and returns the rendered result.","ref":"Phoenix.LiveViewTest.html#render_patch/2","title":"Phoenix.LiveViewTest.render_patch/2","type":"function"},{"doc":"Sends a form submit event given by element and returns the rendered result.The element is created with element/3 and must point to a single element on the page with a phx-submit attribute in it. The event name given set on phx-submit is then sent to the appropriate LiveView (or component if phx-target is set accordingly). All phx-value-* entries in the element are sent as values. Extra values, including hidden input fields, can be given with the value argument.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert view |&gt; element(&quot;form&quot;) |&gt; render_submit(%{deg: 123, avatar: upload}) =~ &quot;123 exceeds limits&quot;To submit a form along with some with hidden input values:assert view |&gt; form(&quot;#term&quot;, user: %{name: &quot;hello&quot;}) |&gt; render_submit(%{user: %{&quot;hidden_field&quot; =&gt; &quot;example&quot;}}) =~ &quot;Name updated&quot;","ref":"Phoenix.LiveViewTest.html#render_submit/2","title":"Phoenix.LiveViewTest.render_submit/2","type":"function"},{"doc":"Sends a form submit event to the view and returns the rendered result.It returns the contents of the whole LiveView or an {:error, redirect} tuple.Examples{:ok, view, html} = live(conn, &quot;/thermo&quot;) assert html =~ &quot;The temp is: 30℉&quot; assert render_submit(view, :refresh, %{deg: 32}) =~ &quot;The temp is: 32℉&quot;","ref":"Phoenix.LiveViewTest.html#render_submit/3","title":"Phoenix.LiveViewTest.render_submit/3","type":"function"},{"doc":"Peforms an upload of a file input and renders the result.See file_input/4 for details on building a file input.ExamplesGiven the following LiveView template:&lt;%= for entry &lt;- @uploads.avatar.entries %&gt; &lt;%=entry.name %&gt;: &lt;%= entry.progress %&gt;% &lt;% end %&gt;Your test case can assert the uploaded content:avatar = file_input(lv, &quot;my-form-id&quot;, :avatar, %{ last_modified: 1_594_171_879_000, name: &quot;myfile.jpeg&quot;, content: File.read!(&quot;myfile.jpg&quot;), size: 1_396_009, type: &quot;image/jpeg&quot; }) assert render_upload(avatar, &quot;foo.jpeg&quot;) =~ &quot;100%&quot;By default, the entire file is chunked to the server, but an optional percentage to chunk can be passed to test chunk-by-chunk uploads:assert render_upload(avatar, &quot;foo.jpeg&quot;, 49) =~ &quot;49%&quot; assert render_upload(avatar, &quot;foo.jpeg&quot;, 51) =~ &quot;100%&quot;","ref":"Phoenix.LiveViewTest.html#render_upload/3","title":"Phoenix.LiveViewTest.render_upload/3","type":"function"},{"doc":"The struct representing an upload.","ref":"Phoenix.LiveView.UploadConfig.html","title":"Phoenix.LiveView.UploadConfig","type":"module"},{"doc":"","ref":"Phoenix.LiveView.UploadConfig.html#t:t/0","title":"Phoenix.LiveView.UploadConfig.t/0","type":"type"},{"doc":"The struct representing an upload entry.","ref":"Phoenix.LiveView.UploadEntry.html","title":"Phoenix.LiveView.UploadEntry","type":"module"},{"doc":"","ref":"Phoenix.LiveView.UploadEntry.html#t:t/0","title":"Phoenix.LiveView.UploadEntry.t/0","type":"type"},{"doc":"The struct returned by Phoenix.LiveViewTest.element/3.The following public fields represent the element:selector - The query selectortext_filter - The text to further filter the elementSee the Phoenix.LiveViewTest documentation for usage.","ref":"Phoenix.LiveViewTest.Element.html","title":"Phoenix.LiveViewTest.Element","type":"module"},{"doc":"The struct returned by Phoenix.LiveViewTest.file_input/4.The following public fields represent the element:selector - The query selectorentries - The list of selected file entriesSee the Phoenix.LiveViewTest documentation for usage.","ref":"Phoenix.LiveViewTest.Upload.html","title":"Phoenix.LiveViewTest.Upload","type":"module"},{"doc":"The struct for testing LiveViews.The following public fields represent the LiveView:id - The DOM id of the LiveViewmodule - The module of the running LiveViewpid - The Pid of the running LiveViewendpoint - The endpoint for the LiveViewSee the Phoenix.LiveViewTest documentation for usage.","ref":"Phoenix.LiveViewTest.View.html","title":"Phoenix.LiveViewTest.View","type":"module"},{"doc":"The struct returned by components in .leex templates.This component is never meant to be output directly into the template. It should always be handled by the diffing algorithm.","ref":"Phoenix.LiveView.Component.html","title":"Phoenix.LiveView.Component","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Component.html#t:t/0","title":"Phoenix.LiveView.Component.t/0","type":"type"},{"doc":"The struct returned by for-comprehensions in .leex templates.See a description about its fields and use cases in Phoenix.LiveView.Engine docs.","ref":"Phoenix.LiveView.Comprehension.html","title":"Phoenix.LiveView.Comprehension","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Comprehension.html#t:t/0","title":"Phoenix.LiveView.Comprehension.t/0","type":"type"},{"doc":"The .leex (Live EEx) template engine that tracks changes.In the documentation below, we will explain how it works internally. For user-facing documentation, see Phoenix.LiveView.Phoenix.LiveView.RenderedWhenever you render a .leex template, it returns a Phoenix.LiveView.Rendered structure. This structure has three fields: :static, :dynamic and :fingerprint.The :static field is a list of literal strings. This allows the Elixir compiler to optimize this list and avoid allocating its strings on every render.The :dynamic field contains a function that takes a boolean argument (see &quot;Tracking changes&quot; below), and returns a list of dynamic content. Each element in the list is either one of:iodata - which is the dynamic contentnil - the dynamic content did not changeanother Phoenix.LiveView.Rendered struct, see &quot;Nesting and fingerprinting&quot; belowa Phoenix.LiveView.Comprehension struct, see &quot;Comprehensions&quot; belowa Phoenix.LiveView.Component struct, see &quot;Component&quot; belowWhen you render a .leex template, you can convert the rendered structure to iodata by alternating the static and dynamic fields, always starting with a static entry followed by a dynamic entry. The last entry will always be static too. So the following structure:%Phoenix.LiveView.Rendered{ static: [&quot;foo&quot;, &quot;bar&quot;, &quot;baz&quot;], dynamic: fn track_changes? -&gt; [&quot;left&quot;, &quot;right&quot;] end }Results in the following content to be sent over the wire as iodata:[&quot;foo&quot;, &quot;left&quot;, &quot;bar&quot;, &quot;right&quot;, &quot;baz&quot;]This is also what calling Phoenix.HTML.Safe.to_iodata/1 with a Phoenix.LiveView.Rendered structure returns.Of course, the benefit of .leex templates is exactly that you do not need to send both static and dynamic segments every time. So let's talk about tracking changes.Tracking changesBy default, a .leex template does not track changes. Change tracking can be enabled by including a changed map in the assigns with the key __changed__ and passing true to the dynamic parts. The map should contain the name of any changed field as key and the boolean true as value. If a field is not listed in :changed, then it is always considered unchanged.If a field is unchanged and .leex believes a dynamic expression no longer needs to be computed, its value in the dynamic list will be nil. This information can be leveraged to avoid sending data to the client.Nesting and fingerprintingPhoenix.LiveView also tracks changes across .leex templates. Therefore, if your view has this:&lt;%= render &quot;form.html&quot;, assigns %&gt;Phoenix will be able to track what is static and dynamic across templates, as well as what changed. A rendered nested .leex template will appear in the dynamic list as another Phoenix.LiveView.Rendered structure, which must be handled recursively.However, because the rendering of live templates can be dynamic in itself, it is important to distinguish which .leex template was rendered. For example, imagine this code:&lt;%= if something?, do: render(&quot;one.html&quot;, assigns), else: render(&quot;other.html&quot;, assigns) %&gt;To solve this, all Phoenix.LiveView.Rendered structs also contain a fingerprint field that uniquely identifies it. If the fingerprints are equal, you have the same template, and therefore it is possible to only transmit its changes.ComprehensionsAnother optimization done by .leex templates is to track comprehensions. If your code has this:&lt;%= for point &lt;- @points do %&gt; x: &lt;%= point.x %&gt; y: &lt;%= point.y %&gt; &lt;% end %&gt;Instead of rendering all points with both static and dynamic parts, it returns a Phoenix.LiveView.Comprehension struct with the static parts, that are shared across all points, and a list of dynamics to be interpolated inside the static parts. If @points is a list with %{x: 1, y: 2} and %{x: 3, y: 4}, the above expression would return:%Phoenix.LiveView.Comprehension{ static: [&quot;\\n x: &quot;, &quot;\\n y: &quot;, &quot;\\n&quot;], dynamics: [ [&quot;1&quot;, &quot;2&quot;], [&quot;3&quot;, &quot;4&quot;] ] }This allows .leex templates to drastically optimize the data sent by comprehensions, as the static parts are emitted only once, regardless of the number of items.The list of dynamics is always a list of iodatas or components, as we don't perform change tracking inside the comprehensions themselves. Similarly, comprehensions do not have fingerprints because they are only optimized at the root, so conditional evaluation, as the one seen in rendering, is not possible. The only possible outcome for a dynamic field that returns a comprehension is nil.Components.leex also supports stateful components. Since they are stateful, they are always handled lazily by the diff algorithm.","ref":"Phoenix.LiveView.Engine.html","title":"Phoenix.LiveView.Engine","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Engine.html#nested_changed_assign?/3","title":"Phoenix.LiveView.Engine.nested_changed_assign?/3","type":"function"},{"doc":"The struct returned by .leex templates.See a description about its fields and use cases in Phoenix.LiveView.Engine docs.","ref":"Phoenix.LiveView.Rendered.html","title":"Phoenix.LiveView.Rendered","type":"module"},{"doc":"","ref":"Phoenix.LiveView.Rendered.html#t:t/0","title":"Phoenix.LiveView.Rendered.t/0","type":"type"},{"doc":"InstallationNote: Phoenix v1.5 comes with built-in support for LiveView apps. Just create your application with mix phx.new my_app --live. If you are using earlier Phoenix versions or your app already exists, keep on reading.The instructions below will serve if you are installing the latest stable version from Hex. To start using LiveView, add one of the following dependencies to your mix.exs and run mix deps.get.If installing from Hex, use the latest version from there:def deps do [ {:phoenix_live_view, &quot;~&gt; 0.15.3&quot;}, {:floki, &quot;&gt;= 0.27.0&quot;, only: :test} ] endIf you want the latest features, install from GitHub:def deps do [ {:phoenix_live_view, github: &quot;phoenixframework/phoenix_live_view&quot;}, {:floki, &quot;&gt;= 0.27.0&quot;, only: :test} ]Once installed, update your endpoint's configuration to include a signing salt. You can generate a signing salt by running mix phx.gen.secret 32:# config/config.exs config :my_app, MyAppWeb.Endpoint, live_view: [signing_salt: &quot;SECRET_SALT&quot;]Next, add the following imports to your web file in lib/my_app_web.ex:# lib/my_app_web.ex def controller do quote do ... import Phoenix.LiveView.Controller end end def view do quote do ... import Phoenix.LiveView.Helpers end end def router do quote do ... import Phoenix.LiveView.Router end endThen add the Phoenix.LiveView.Router.fetch_live_flash/2 plug to your browser pipeline, in place of :fetch_flash:# lib/my_app_web/router.ex pipeline :browser do ... plug :fetch_session - plug :fetch_flash + plug :fetch_live_flash endNext, expose a new socket for LiveView updates in your app's endpoint module.# lib/my_app_web/endpoint.ex defmodule MyAppWeb.Endpoint do use Phoenix.Endpoint # ... socket &quot;/live&quot;, Phoenix.LiveView.Socket, websocket: [connect_info: [session: @session_options]] # ... endWhere @session_options are the options given to plug Plug.Session by using a module attribute. If you don't have a @session_options in your endpoint yet, here is how to create one:Find plug Plug.Session in your endpoint.ex plug Plug.Session store: :cookie, key: &quot;_my_app_key&quot;, signing_salt: &quot;somesigningsalt&quot;Move the options to a module attribute at the top of your file: @session_options [ store: :cookie, key: &quot;_my_app_key&quot;, signing_salt: &quot;somesigningsalt&quot; ]Change the plug Plug.Session to use that attribute: plug Plug.Session, @session_optionsAdd LiveView NPM dependencies to your assets/package.json. For a regular project, do:{ &quot;dependencies&quot;: { &quot;phoenix&quot;: &quot;file:../deps/phoenix&quot;, &quot;phoenix_html&quot;: &quot;file:../deps/phoenix_html&quot;, &quot;phoenix_live_view&quot;: &quot;file:../deps/phoenix_live_view&quot; } }However, if you're adding phoenix_live_view to an umbrella project, the dependency paths should be modified appropriately:{ &quot;dependencies&quot;: { &quot;phoenix&quot;: &quot;file:../../../deps/phoenix&quot;, &quot;phoenix_html&quot;: &quot;file:../../../deps/phoenix_html&quot;, &quot;phoenix_live_view&quot;: &quot;file:../../../deps/phoenix_live_view&quot; } }Then install the new NPM dependency:npm install --prefix assetsIf you had previously installed phoenix_live_view and want to get the latest javascript, then force an install with:npm install --force phoenix_live_view --prefix assetsFinally, ensure you have placed a CSRF meta tag inside the &lt;head&gt; tag in your layout (lib/my_app_web/templates/layout/root.html.leex), before app.js is included like so:&lt;%= csrf_meta_tag() %&gt; &lt;script defer type=&quot;text/javascript&quot; src=&quot;&lt;%= Routes.static_path(@conn, &quot;/js/app.js&quot;) %&gt;&quot;&gt;&lt;/script&gt;and enable connecting to a LiveView socket in your app.js file.// assets/js/app.js import {Socket} from &quot;phoenix&quot; import LiveSocket from &quot;phoenix_live_view&quot; let csrfToken = document.querySelector(&quot;meta[name='csrf-token']&quot;).getAttribute(&quot;content&quot;) let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {params: {_csrf_token: csrfToken}}) // Connect if there are any LiveViews on the page liveSocket.connect() // Expose liveSocket on window for web console debug logs and latency simulation: // &gt;&gt; liveSocket.enableDebug() // &gt;&gt; liveSocket.enableLatencySim(1000) // The latency simulator is enabled for the duration of the browser session. // Call disableLatencySim() to disable: // &gt;&gt; liveSocket.disableLatencySim() window.liveSocket = liveSocket","ref":"installation.html","title":"Installation","type":"extras"},{"doc":"LiveView does not use the default app layout. Instead, you typically call put_root_layout in your router to specify a layout that is used by both &quot;regular&quot; views and live views. In your router, do:pipeline :browser do ... plug :put_root_layout, {MyAppWeb.LayoutView, :root} ... endThe layout given to put_root_layout must use &lt;%= @inner_content %&gt; instead of &lt;%= render(@view_module, @view_template, assigns) %&gt;. It is typically very barebones, with mostly &lt;head&gt; and &lt;body&gt; tags. For example:&lt;!DOCTYPE html&gt; &lt;html lang=&quot;en&quot;&gt; &lt;head&gt; &lt;%= csrf_meta_tag() %&gt; &lt;%= live_title_tag assigns[:page_title] || &quot;MyApp&quot; %&gt; &lt;link rel=&quot;stylesheet&quot; href=&quot;&lt;%= Routes.static_path(@conn, &quot;/css/app.css&quot;) %&gt;&quot;/&gt; &lt;script defer type=&quot;text/javascript&quot; src=&quot;&lt;%= Routes.static_path(@conn, &quot;/js/app.js&quot;) %&gt;&quot;&gt;&lt;/script&gt; &lt;/head&gt; &lt;body&gt; &lt;%= @inner_content %&gt; &lt;/body&gt; &lt;/html&gt;Once you have specified a root layout, &quot;app.html.eex&quot; will be rendered within your root layout for all non-LiveViews. You may also optionally define a &quot;live.html.leex&quot; layout to be used across all LiveViews, as we will describe in the next section.","ref":"installation.html#layouts","title":"Installation - Layouts","type":"extras"},{"doc":"While the above instructions are enough to install LiveView in a Phoenix app, if you want to use the phx.gen.live generators that come as part of Phoenix v1.5, you need to do one more change, as those generators assume your application was created with mix phx.new --live.The change is to define the live_view and live_component functions in your my_app_web.ex file, while refactoring the view function. At the end, they will look like this: def view do quote do use Phoenix.View, root: &quot;lib/&lt;%= lib_web_name %&gt;/templates&quot;, namespace: &lt;%= web_namespace %&gt; # Import convenience functions from controllers import Phoenix.Controller, only: [get_flash: 1, get_flash: 2, view_module: 1, view_template: 1] # Include shared imports and aliases for views unquote(view_helpers()) end end def live_view do quote do use Phoenix.LiveView, layout: {&lt;%= web_namespace %&gt;.LayoutView, &quot;live.html&quot;} unquote(view_helpers()) end end def live_component do quote do use Phoenix.LiveComponent unquote(view_helpers()) end end defp view_helpers do quote do # Use all HTML functionality (forms, tags, etc) use Phoenix.HTML # Import LiveView helpers (live_render, live_component, live_patch, etc) import Phoenix.LiveView.Helpers # Import basic rendering functionality (render, render_layout, etc) import Phoenix.View import MyAppWeb.ErrorHelpers import MyAppWeb.Gettext alias MyAppWeb.Router.Helpers, as: Routes end endNote that LiveViews are automatically configured to use a &quot;live.html.leex&quot; layout in this line:use Phoenix.LiveView, layout: {&lt;%= web_namespace %&gt;.LayoutView, &quot;live.html&quot;}&quot;root.html.leex&quot; is shared by regular and live views, &quot;app.html.eex&quot; is rendered inside the root layout for regular views, and &quot;live.html.leex&quot; is rendered inside the root layout for LiveViews. &quot;live.html.leex&quot; typically starts out as a copy of &quot;app.html.eex&quot;, but using the @socket assign instead of @conn. Check the Live Layouts guide for more information.","ref":"installation.html#phx-gen-live-support","title":"Installation - phx.gen.live support","type":"extras"},{"doc":"If you want to show a progress bar as users perform live actions, we recommend using topbar.First add topbar as a dependency:$ npm install --prefix assets --save topbarThen customize LiveView to use it in your assets/js/app.js, right before the liveSocket.connect() call:import topbar from &quot;topbar&quot; // Show progress bar on live navigation and form submits topbar.config({barColors: {0: &quot;#29d&quot;}, shadowColor: &quot;rgba(0, 0, 0, .3)&quot;}) window.addEventListener(&quot;phx:page-loading-start&quot;, info =&gt; topbar.show()) window.addEventListener(&quot;phx:page-loading-stop&quot;, info =&gt; topbar.hide())","ref":"installation.html#progress-animation","title":"Installation - Progress animation","type":"extras"},{"doc":"By convention your LiveView modules and leex templates should be placed in lib/my_web_app/live/ directory.","ref":"installation.html#location-for-liveview-modules","title":"Installation - Location for LiveView modules","type":"extras"},{"doc":"Assigns and LiveEEx templatesAll of the data in a LiveView is stored in the socket as assigns. The Phoenix.LiveView.assign/2 and Phoenix.LiveView.assign/3 functions help store those values. Those values can be accessed in the LiveView as socket.assigns.name but they are accessed inside LiveView templates as @name.Phoenix.LiveView's built-in templates are identified by the .leex extension (Live EEx) or ~L sigil. They are similar to regular .eex templates except they are designed to minimize the amount of data sent over the wire by splitting static and dynamic parts and tracking changes.When you first render a .leex template, it will send all of the static and dynamic parts of the template to the client. After that, any change you do on the server will now send only the dynamic parts, and only if those parts have changed.The tracking of changes is done via assigns. Imagine this template:&lt;h1&gt;&lt;%= expand_title(@title) %&gt;&lt;/h1&gt;If the @title assign changes, then LiveView will execute expand_title(@title) and send the new content. If @title is the same, nothing is executed and nothing is sent.Change tracking also works when accessing map/struct fields. Take this template:&lt;div id=&quot;user_&lt;%= @user.id %&gt;&quot;&gt; &lt;%= @user.name %&gt; &lt;/div&gt;If the @user.name changes but @user.id doesn't, then LiveView will re-render only @user.name and it will not execute or resend @user.id at all.The change tracking also works when rendering other templates as long as they are also .leex templates:&lt;%= render &quot;child_template.html&quot;, assigns %&gt;The assign tracking feature also implies that you MUST avoid performing direct operations in the template. For example, if you perform a database query in your template:&lt;%= for user &lt;- Repo.all(User) do %&gt; &lt;%= user.name %&gt; &lt;% end %&gt;Then Phoenix will never re-render the section above, even if the number of users in the database changes. Instead, you need to store the users as assigns in your LiveView before it renders the template:assign(socket, :users, Repo.all(User))Generally speaking, data loading should never happen inside the template, regardless if you are using LiveView or not. The difference is that LiveView enforces this best practice.","ref":"assigns-eex.html","title":"Assigns and LiveEEx templates","type":"extras"},{"doc":"There are two common pitfalls to keep in mind when using the ~L sigil or .leex templates.When it comes to do/end blocks, change tracking is supported only on blocks given to Elixir's basic constructs, such as if, case, for, and friends. If the do/end block is given to a library function or user function, such as content_tag, change tracking won't work. For example, imagine the following template that renders a div:&lt;%= content_tag :div, id: &quot;user_\#{@id}&quot; do %&gt; &lt;%= @name %&gt; &lt;%= @description %&gt; &lt;% end %&gt;LiveView knows nothing about content_tag, which means the whole div will be sent whenever any of the assigns change. This can be easily fixed by writing the HTML directly:&lt;div id=&quot;user_&lt;%= @id %&gt;&quot;&gt; &lt;%= @name %&gt; &lt;%= @description %&gt; &lt;/div&gt;Another pitfall of .leex templates is related to variables. Due to the scope of variables, LiveView has to disable change tracking whenever variables are used in the template, with the exception of variables introduced by Elixir basic case, for, and other block constructs. Therefore, you must avoid code like this in your LiveEEx:&lt;% some_var = @x + @y %&gt; &lt;%= some_var %&gt;Instead, use a function:&lt;%= sum(@x, @y) %&gt;Similarly, do not define variables at the top of your render function:def render(assigns) do sum = assigns.x + assigns.y ~L&quot;&quot;&quot; &lt;%= sum %&gt; &quot;&quot;&quot; endInstead explicitly precompute the assign in your LiveView, outside of render:assign(socket, sum: socket.assigns.x + socket.assigns.y)Generally speaking, avoid accessing variables inside LiveViews. This also applies to the assigns variable, except when rendering another .leex template. In such cases, it is ok to pass the whole assigns, as LiveView will continue to perform change tracking in the called template:&lt;%= render &quot;sidebar.html&quot;, assigns %&gt;Similarly, variables introduced by Elixir's block constructs are fine. For example, accessing the post variable defined by the comprehension below works as expected:&lt;%= for post &lt;- @posts do %&gt; ... &lt;% end %&gt;As are the variables matched defined in a case or cond:&lt;%= cond do %&gt; &lt;% is_nil(@post) -&gt; %&gt; ... &lt;% @post -&gt; %&gt; ... &lt;% end %&gt;To sum up:Avoid passing block expressions to library and custom functionsNever do anything on def render(assigns) besides rendering a template or invoking the ~L sigilAvoid defining local variables, except within for, case, and friends","ref":"assigns-eex.html#liveeex-pitfalls","title":"Assigns and LiveEEx templates - LiveEEx pitfalls","type":"extras"},{"doc":"Error and exception handlingAs with any other Elixir code, exceptions may happen during the LiveView life-cycle. In this section we will describe how LiveView reacts to errors at different stages.","ref":"error-handling.html","title":"Error and exception handling","type":"extras"},{"doc":"In this section, we will talk about error cases that you expect to happen within your application. For example, a user filling in a form with invalid data is expected. In a LiveView, we typically handle those cases by storing a change in the LiveView state, which causes the LiveView to be re-rendered with the error message.We may also use flash messages for this. For example, imagine you have a page to manage all &quot;Team members&quot; in an organization. However, if there is only one member left in the organization, they should not be allowed to leave. You may want to handle this by using flash messages:if MyApp.Org.leave(socket.assigns.current_org, member) do {:noreply, socket} else {:noreply, put_flash(socket, :error, &quot;last member cannot leave organization&quot;)} endHowever, one may argue that, if the last member of an organization cannot leave it, it may be better to not even show the &quot;Leave&quot; button in the UI when the organization has only one member.Given the button does not appear in the UI, triggering the &quot;leave&quot; when the organization has now only one member is an unexpected scenario. This means we can probably rewrite the code above to:true = MyApp.Org.leave(socket.assigns.current_org, member) {:noreply, socket}If leave returns false by any chance, it will just raise. Or you can even provide a leave! function that raises a specific exception:MyApp.Org.leave!(socket.assigns.current_org, member) {:noreply, socket}However, what will happen with a LiveView in case of exceptions? Let's talk about unexpected scenarios.","ref":"error-handling.html#expected-scenarios","title":"Error and exception handling - Expected scenarios","type":"extras"},{"doc":"Elixir developers tend to write assertive code. This means that, if we expect leave to always return true, we can explicitly match on its result, as we did above:true = MyApp.Org.leave(socket.assigns.current_org, member) {:noreply, socket}If leave fails and returns false, an exception is raised. It is common for Elixir developers to use exceptions for unexpected scenarios in their Phoenix applications.For example, if you are building an application where a user may belong to one or more organizations, when accessing the organization page, you may want to check that the user has access to it like this:organizations_query = Ecto.assoc(socket.assigns.current_user, :organizations) Repo.get!(organizations_query, params[&quot;org_id&quot;])The code above builds a query that returns all organizations that belongs to the current user and then validates that the given &quot;org_id&quot; belongs to the user. If there is no such &quot;org_id&quot; or if the user has no access to it, an Ecto.NotFoundError exception is raised.During a regular controller request, this exception will be converted to a 404 exception and rendered as a custom error page, as detailed here. To understand how a LiveView reacts to exceptions, we need to consider two scenarios: exceptions on mount and during any event.","ref":"error-handling.html#unexpected-scenarios","title":"Error and exception handling - Unexpected scenarios","type":"extras"},{"doc":"Given the code on mount runs both on the initial disconnected render and the connected render, an exception on mount will trigger the following events:Exceptions during disconnected render:An exception on mount is caught and converted to an exception page by Phoenix error views - pretty much like the way it works with controllersExceptions during connected render:An exception on mount will crash the LiveView process - which will be loggedOnce the client has noticed the crash during mount, it will fully reload the pageReloading the page will start a disconnected render, that will cause mount to be invoked again and most likely raise the same exception. Except this time it will be caught and converted to an exception page by Phoenix error viewsIn other words, LiveView will reload the page in case of errors, making it fail as if LiveView was not involved in the rendering in the first place.","ref":"error-handling.html#exceptions-on-mount","title":"Error and exception handling - Exceptions on mount","type":"extras"},{"doc":"If the error happens during an event, the LiveView process will crash. The client will notice the error and remount the LiveView - without reloading the page. This is enough to update the page and show the user the latest information.For example, let's say two users try to leave the organization at the same time. In this case, both of them see the &quot;Leave&quot; button, but our leave function call will succeed only for one of them:true = MyApp.Org.leave(socket.assigns.current_org, member) {:noreply, socket}When the exception raises, the client will remount the LiveView. Once you remount, your code will now notice that there is only one user in the organization and therefore no longer show the &quot;Leave&quot; button. In other words, by remounting, we often update the state of the page, allowing exceptions to be automatically handled.Note that the choice between conditionally checking on the result of the leave function with an if, or simply asserting it returns true, is completely up to you. If the likelihood of everyone leaving the organization at the same time is low, then you may as well treat it as an unexpected scenario. Although other developers will be more comfortable by explicitly handling those cases. In both scenarios, LiveView has you covered.","ref":"error-handling.html#exceptions-on-events-handle_info-handle_event-etc","title":"Error and exception handling - Exceptions on events (handle_info, handle_event, etc)","type":"extras"},{"doc":"Live layoutsNOTE: Make sure you've read the Assigns and LiveEEx templates guide before moving forward.When working with LiveViews, there are usually three layouts to be considered:the root layout - this is a layout used by both LiveView and regular views. This layout typically contains the &lt;html&gt; definition alongside the head and body tags. Any content defined in the root layout will remain the same, even as you live navigate across LiveViews. All LiveViews defined at the router must have a root layout. The root layout is typically declared on the router with put_root_layout and defined as &quot;root.html.eex&quot; in your MyAppWeb.LayoutView. It may also be given via the the :layout option to the router's live macro.the app layout - this is the default application layout which is not included or used by LiveViews. It defaults to &quot;app.html.eex&quot; in your MyAppWeb.LayoutView.the live layout - this is the layout which wraps a LiveView and is rendered as part of the LiveView life-cycle. It must be opt-in by passing the :layout option on use Phoenix.LiveView. It is typically set to &quot;live.html.leex&quot;in your MyAppWeb.LayoutView.Overall, those layouts are found in templates/layout with the following names:* root.html.eex * app.html.eex * live.html.leexAll layouts must call &lt;%= @inner_content %&gt; to inject the content rendered by the layout.The &quot;root&quot; layout is shared by both &quot;app&quot; and &quot;live&quot; layouts. It is rendered only on the initial request and therefore it has access to the @conn assign. The root layout must be defined in your router:plug :put_root_layout, {MyAppWeb.LayoutView, :root}Alternatively, the root layout can be passed individually to the live macro of your live routes:live &quot;/dashboard&quot;, MyAppWeb.Dashboard, layout: {MyAppWeb.LayoutView, :root}The &quot;app&quot; and &quot;live&quot; layouts are often small and similar to each other, but the &quot;app&quot; layout uses the @conn and is used as part of the regular request life-cycle. The &quot;live&quot; layout is part of the LiveView and therefore has direct access to the @socket.For example, you can define a new live.html.leex layout with dynamic content. You must use @inner_content where the output of the actual template will be placed at:&lt;p&gt;&lt;%= live_flash(@flash, :notice) %&gt;&lt;/p&gt; &lt;p&gt;&lt;%= live_flash(@flash, :error) %&gt;&lt;/p&gt; &lt;%= @inner_content %&gt;To use the live layout, update your LiveView to pass the :layout option to use Phoenix.LiveView:use Phoenix.LiveView, layout: {MyAppWeb.LayoutView, &quot;live.html&quot;}If you are using Phoenix v1.5, the layout is automatically set when generating apps with the mix phx.new --live flag.The :layout option on use does not apply to LiveViews rendered within other LiveViews. If you want to render child live views or opt-in to a layout, use :layout as an option in mount: def mount(_params, _session, socket) do socket = assign(socket, new_message_count: 0) {:ok, socket, layout: {MyAppWeb.LayoutView, &quot;live.html&quot;}} endNote: The live layout is always wrapped by the LiveView's :container tag.","ref":"live-layouts.html","title":"Live layouts","type":"extras"},{"doc":"Because the root layout from the Plug pipeline is rendered outside of LiveView, the contents cannot be dynamically changed. The one exception is the &lt;title&gt; of the HTML document. Phoenix LiveView special cases the @page_title assign to allow dynamically updating the title of the page, which is useful when using live navigation, or annotating the browser tab with a notification. For example, to update the user's notification count in the browser's title bar, first set the page_title assign on mount: def mount(_params, _session, socket) do socket = assign(socket, page_title: &quot;Latest Posts&quot;) {:ok, socket} endThen access @page_title in the root layout:&lt;title&gt;&lt;%= @page_title %&gt;&lt;/title&gt;You can also use Phoenix.LiveView.Helpers.live_title_tag/2 to support adding automatic prefix and suffix to the page title when rendered and on subsequent updates:&lt;%= live_title_tag assigns[:page_title] || &quot;Welcome&quot;, prefix: &quot;MyApp – &quot; %&gt;Although the root layout is not updated by LiveView, by simply assigning to page_title, LiveView knows you want the title to be updated:def handle_info({:new_messages, count}, socket) do {:noreply, assign(socket, page_title: &quot;Latest Posts (\#{count} new)&quot;)} endNote: If you find yourself needing to dynamically patch other parts of the base layout, such as injecting new scripts or styles into the &lt;head&gt; during live navigation, then a regular, non-live, page navigation should be used instead. Assigning the @page_title updates the document.title directly, and therefore cannot be used to update any other part of the base layout.","ref":"live-layouts.html#updating-the-html-document-title","title":"Live layouts - Updating the HTML document title","type":"extras"},{"doc":"Live navigationLiveView provides functionality to allow page navigation using the browser's pushState API. With live navigation, the page is updated without a full page reload.You can trigger live navigation in two ways:From the client - this is done by replacing Phoenix.HTML.Link.link/2 by Phoenix.LiveView.Helpers.live_patch/2 or Phoenix.LiveView.Helpers.live_redirect/2From the server - this is done by replacing Phoenix.Controller.redirect/2 calls by Phoenix.LiveView.push_patch/2 or Phoenix.LiveView.push_redirect/2.For example, in a template you may write:&lt;%= live_patch &quot;next&quot;, to: Routes.live_path(@socket, MyLive, @page + 1) %&gt;or in a LiveView:{:noreply, push_patch(socket, to: Routes.live_path(socket, MyLive, page + 1))}The &quot;patch&quot; operations must be used when you want to navigate to the current LiveView, simply updating the URL and the current parameters, without mounting a new LiveView. When patch is used, the handle_params/3 callback is invoked and the minimal set of changes are sent to the client. See the next section for more information.The &quot;redirect&quot; operations must be used when you want to dismount the current LiveView and mount a new one. In those cases, an Ajax request is made to fetch the necessary information about the new LiveView, which is mounted in place of the current one within the current layout. While redirecting, a phx-disconnected class is added to the LiveView, which can be used to indicate to the user a new page is being loaded.At the end of the day, regardless if you invoke link/2, live_patch/2, and live_redirect/2 from the client, or redirect/2, push_patch/2, and push_redirect/2 from the server, the user will end-up on the same page. The difference between those is mostly the amount of data sent over the wire:link/2 and redirect/2 do full page reloadslive_redirect/2 and push_redirect/2 mounts a new LiveView while keeping the current layoutlive_patch/2 and push_patch/2 updates the current LiveView and sends only the minimal diff while also maintaining the scroll positionAn easy rule of thumb is to stick with live_redirect/2 and push_redirect/2 and use the patch helpers only in the cases where you want to minimize the amount of data sent when navigating within the same LiveView (for example, if you want to change the sorting of a table while also updating the URL).","ref":"live-navigation.html","title":"Live navigation","type":"extras"},{"doc":"The handle_params/3 callback is invoked after mount/3 and before the initial render. It is also invoked every time live_patch/2 or push_patch/2 are used. It receives the request parameters as first argument, the url as second, and the socket as third.For example, imagine you have a UserTable LiveView to show all users in the system and you define it in the router as:live &quot;/users&quot;, UserTableNow to add live sorting, you could do:&lt;%= live_patch &quot;Sort by name&quot;, to: Routes.live_path(@socket, UserTable, %{sort_by: &quot;name&quot;}) %&gt;When clicked, since we are navigating to the current LiveView, handle_params/3 will be invoked. Remember you should never trust the received params, so you must use the callback to validate the user input and change the state accordingly:def handle_params(params, _uri, socket) do socket = case params[&quot;sort_by&quot;] do sort_by when sort_by in ~w(name company) -&gt; assign(socket, sort_by: sort) _ -&gt; socket end {:noreply, load_users(socket)} endAs with other handle_* callback, changes to the state inside handle_params/3 will trigger a server render.Note the parameters given to handle_params/3 are the same as the ones given to mount/3. So how do you decide which callback to use to load data? Generally speaking, data should always be loaded on mount/3, since mount/3 is invoked once per LiveView life-cycle. Only the params you expect to be changed via live_patch/2 or push_patch/2 must be loaded on handle_params/3.For example, imagine you have a blog. The URL for a single post is: &quot;/blog/posts/:post_id&quot;. In the post page, you have comments and they are paginated. You use live_patch/2 to update the shown comments every time the user paginates, updating the URL to &quot;/blog/posts/:post_id?page=X&quot;. In this example, you will access &quot;post_id&quot; on mount/3 and the page of comments on handle_params/3.Furthermore, it is very important to not access the same parameters on both mount/3 and handle_params/3. For example, do NOT do this:def mount(%{&quot;post_id&quot; =&gt; post_id}, session, socket) do # do something with post_id end def handle_params(%{&quot;post_id&quot; =&gt; post_id, &quot;page&quot; =&gt; page}, url, socket) do # do something with post_id and page endIf you do that, because mount/3 is called once and handle_params/3 multiple times, the &quot;post_id&quot; read on mount can get out of sync with the one in handle_params/3. So once a parameter is read on mount, it should not be read elsewhere. Instead, do this:def mount(%{&quot;post_id&quot; =&gt; post_id}, session, socket) do # do something with post_id end def handle_params(%{&quot;sort_by&quot; =&gt; sort_by}, url, socket) do post_id = socket.assigns.post.id # do something with sort_by end","ref":"live-navigation.html#handle_params-3","title":"Live navigation - handle_params/3","type":"extras"},{"doc":"LiveView also allows the current browser URL to be replaced. This is useful when you want certain events to change the URL but without polluting the browser's history. This can be done by passing the replace: true option to any of the navigation helpers.","ref":"live-navigation.html#replace-page-address","title":"Live navigation - Replace page address","type":"extras"},{"doc":"LiveView allows you to have multiple LiveViews in the same page by calling Phoenix.LiveView.Helpers.live_render/3 in your templates. However, only the LiveViews defined directly in your router can use the &quot;Live Navigation&quot; functionality described here. This is important because LiveViews work closely with your router, guaranteeing you can only navigate to known routes.","ref":"live-navigation.html#multiple-liveviews-in-the-same-page","title":"Live navigation - Multiple LiveViews in the same page","type":"extras"},{"doc":"Security considerations of the LiveView modelAs we have seen, LiveView begins its life-cycle as a regular HTTP request. Then a stateful connection is established. Both the HTTP request and the stateful connection receives the client data via parameters and session. This means that any session validation must happen both in the HTTP request and the stateful connection.","ref":"security-model.html","title":"Security considerations of the LiveView model","type":"extras"},{"doc":"For example, if you perform user authentication and confirmation on every HTTP request via Plugs, such as this:plug :ensure_user_authenticated plug :ensure_user_confirmedThen the mount/3 callback of your LiveView should execute those same verifications:def mount(params, %{&quot;user_id&quot; =&gt; user_id} = _session, socket) do socket = assign(socket, current_user: Accounts.get_user!(user_id)) socket = if socket.assigns.current_user.confirmed_at do socket else redirect(socket, to: &quot;/login&quot;) end {:ok, socket} endGiven almost all mount/3 actions in your application will have to perform these exact steps, we recommend creating a function called assign_defaults/2 or similar, putting it in a new module like MyAppWeb.LiveHelpers, and modifying lib/my_app_web.ex so all LiveViews automatically import it:def live_view do quote do # ...other stuff... import MyAppWeb.LiveHelpers end endThen make sure to call it in every LiveView's mount/3:def mount(params, session, socket) do {:ok, assign_defaults(session, socket)} endWhere MyAppWeb.LiveHelpers can be something like:defmodule MyAppWeb.LiveHelpers do import Phoenix.LiveView def assign_defaults(%{&quot;user_id&quot; =&gt; user_id}, socket) do socket = assign(socket, current_user: Accounts.get_user!(user_id)) if socket.assigns.current_user.confirmed_at do socket else redirect(socket, to: &quot;/login&quot;) end end endOne possible concern in this approach is that in regular HTTP requests the current user will be fetched twice: once in the HTTP request and again on mount/3. You can address this by using the assign_new/3 function, that will reuse any of the connection assigns from the HTTP request:def assign_defaults(%{&quot;user_id&quot; =&gt; user_id}, socket) do socket = assign_new(socket, :current_user, fn -&gt; Accounts.get_user!(user_id) end) if socket.assigns.current_user.confirmed_at do socket else redirect(socket, to: &quot;/login&quot;) end end","ref":"security-model.html#mounting-considerations","title":"Security considerations of the LiveView model - Mounting considerations","type":"extras"},{"doc":"It is also important to keep in mind that LiveViews are stateful. Therefore, if you load any data on mount/3 and the data itself changes, the data won't be automatically propagated to the LiveView, unless you broadcast those events with Phoenix.PubSub.Generally speaking, the simplest and safest approach is to perform authorization whenever there is an action. For example, imagine that you have a LiveView for a &quot;Blog&quot;, and only editors can edit posts. Therefore, it is best to validate the user is an editor on mount and on every event:def mount(%{&quot;post_id&quot; =&gt; post_id}, session, socket) do socket = assign_defaults(session, socket) post = Blog.get_post_for_user!(socket.assigns.current_user, post_id) {:ok, assign(socket, post: post)} end def handle_event(&quot;update_post&quot;, params, socket) do updated_post = Blog.update_post(socket.assigns.current_user, socket.assigns.post, params) {:noreply, assign(socket, post: updated_post)} endIn the example above, the Blog context receives the user on both get and update operations, and always validates accordingly that the user has access, raising an error otherwise.","ref":"security-model.html#events-considerations","title":"Security considerations of the LiveView model - Events considerations","type":"extras"},{"doc":"Another security consideration is how to disconnect all instances of a given live user. For example, imagine the user logs outs, its account is terminated, or any other reason.Luckily, it is possible to identify all LiveView sockets by setting a live_socket_id in the session. For example, when signing in a user, you could do:conn |&gt; put_session(:current_user_id, user.id) |&gt; put_session(:live_socket_id, &quot;users_socket:\#{user.id}&quot;)Now all LiveView sockets will be identified and listening to the given live_socket_id. You can disconnect all live users identified by said ID by broadcasting on the topic:MyAppWeb.Endpoint.broadcast(&quot;users_socket:\#{user.id}&quot;, &quot;disconnect&quot;, %{})Once a LiveView is disconnected, the client will attempt to reestablish the connection, re-executing the mount/3 callback. In this case, if the user is no longer logged in or it no longer has access to its current resource, mount/3 will fail and the user will be redirected to the proper page.This is the same mechanism provided by Phoenix.Channels. Therefore, if your application uses both channels and LiveViews, you can use the same technique to disconnect any stateful connection.","ref":"security-model.html#disconnecting-all-instances-of-a-given-live-user","title":"Security considerations of the LiveView model - Disconnecting all instances of a given live user","type":"extras"},{"doc":"TelemetryLiveView currently exposes the following telemetry events:[:phoenix, :live_view, :mount, :start] - Dispatched by a Phoenix.LiveView immediately before mount/3 is invoked.Measurement:%{system_time: System.monotonic_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, params: unsigned_params | :not_mounted_at_router, session: map }[:phoenix, :live_view, :mount, :stop] - Dispatched by a Phoenix.LiveView when the mount/3 callback completes successfully.Measurement:%{duration: native_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, params: unsigned_params | :not_mounted_at_router, session: map }[:phoenix, :live_view, :mount, :exception] - Dispatched by a Phoenix.LiveView when an exception is raised in the mount/3 callback.Measurement: %{duration: native_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, kind: atom, reason: term, params: unsigned_params | :not_mounted_at_router, session: map }[:phoenix, :live_view, :handle_params, :start] - Dispatched by a Phoenix.LiveView immediately before handle_params/3 is invoked.Measurement:%{system_time: System.monotonic_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, params: unsigned_params, uri: String.t() }[:phoenix, :live_view, :handle_params, :stop] - Dispatched by a Phoenix.LiveView when the handle_params/3 callback completes successfully.Measurement:%{duration: native_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, params: unsigned_params, uri: String.t() }[:phoenix, :live_view, :handle_params, :exception] - Dispatched by a Phoenix.LiveView when the when an exception is raised in the handle_params/3 callback.Measurement:%{duration: native_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, kind: atom, reason: term, params: unsigned_params, uri: String.t() }[:phoenix, :live_view, :handle_event, :start] - Dispatched by a Phoenix.LiveView immediately before handle_event/3 is invoked.Measurement:%{system_time: System.monotonic_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, event: String.t(), params: unsigned_params }[:phoenix, :live_view, :handle_event, :stop] - Dispatched by a Phoenix.LiveView when the handle_event/3 callback completes successfully.Measurement:%{duration: native_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, event: String.t(), params: unsigned_params }[:phoenix, :live_view, :handle_event, :exception] - Dispatched by a Phoenix.LiveView when an exception is raised in the handle_event/3 callback.Measurement:%{duration: native_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, kind: atom, reason: term, event: String.t(), params: unsigned_params }[:phoenix, :live_component, :handle_event, :start] - Dispatched by a Phoenix.LiveComponent immediately before handle_event/3 is invoked.Measurement:%{system_time: System.monotonic_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, component: atom, event: String.t(), params: unsigned_params }[:phoenix, :live_component, :handle_event, :stop] - Dispatched by a Phoenix.LiveComponent when the handle_event/3 callback completes successfully.Measurement:%{duration: native_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, component: atom, event: String.t(), params: unsigned_params }[:phoenix, :live_component, :handle_event, :exception] - Dispatched by a Phoenix.LiveComponent when an exception is raised in the handle_event/3 callback.Measurement:%{duration: native_time}Metadata:%{ socket: Phoenix.LiveView.Socket.t, kind: atom, reason: term, component: atom, event: String.t(), params: unsigned_params }","ref":"telemetry.html","title":"Telemetry","type":"extras"},{"doc":"UploadsLiveView supports interactive file uploads with progress for both direct to server uploads as well as external direct-to-cloud uploads on the client.Uploads are enabled by using Phoenix.LiveView.allow_upload/3 and specifying the constraints, such as accepted file types, max file size, number of maximum selected entries, etc. When the client selects file(s), the file metadata is automatically validated against the allow_upload specification. Uploads are populated in an @uploads assign in the socket, granting reactive based templates that automatically update with progress, error information, etc.The complete upload flow is as follows:","ref":"uploads.html","title":"Uploads","type":"extras"},{"doc":"You enable an upload, typically on mount, via allow_upload/3:@impl Phoenix.LiveView def mount(_params, _session, socket) do {:ok, socket |&gt; assign(:uploaded_files, []) |&gt; allow_upload(:avatar, accept: ~w(.jpg .jpeg), max_entries: 2)} end","ref":"uploads.html#allow-uploads","title":"Uploads - Allow uploads","type":"extras"},{"doc":"Use the Phoenix.LiveView.Helpers.live_file_input/2 file input generator to render a file input for the upload. The generator has full support for multiple=true, and the attribute will automatically be set if :max_entries is greater than 1 in the allow_upload/3 spec.Within the template, you render each upload entry. The entry struct contains all the information about the upload, including progress, name, errors, etc.For example:&lt;%= for entry &lt;- @uploads.avatar.entries do %&gt; &lt;%= entry.client_name %&gt; - &lt;%= entry.progress %&gt;% &lt;% end %&gt; &lt;form phx-submit=&quot;save&quot; phx-change=&quot;validate&quot;&gt; &lt;%= live_file_input @uploads.avatar %&gt; &lt;button type=&quot;submit&quot;&gt;Upload&lt;/button&gt; &lt;/form&gt;You must bind phx-submit and phx-change on the form.Reactive updates to the template will occur as the end-user interacts with the file input. Validation will occur based on any conditions you have specified in allow_upload/3 but you must implement at least a minimal callback:def handle_event(&quot;validate&quot;, _params, socket) do {:noreply, socket} end","ref":"uploads.html#render-reactive-elements","title":"Uploads - Render reactive elements","type":"extras"},{"doc":"When the end-user submits a form containing a live_file_input/2, the JavaScript client first uploads the file(s) before invoking the callback for the form's phx-submit event.Within the callback for the phx-submit event, you invoke the Phoenix.LiveView.consume_uploaded_entries/3 function to process the completed uploads, persisting the relevant upload data alongside the form data:def handle_event(&quot;save&quot;, _params, socket) do uploaded_files = consume_uploaded_entries(socket, :avatar, fn %{path: path}, _entry -&gt; dest = Path.join(&quot;priv/static/uploads&quot;, Path.basename(path)) File.cp!(path, dest) Routes.static_path(socket, &quot;/uploads/\#{Path.basename(dest)}&quot;) end) {:noreply, update(socket, :uploaded_files, &amp;(&amp;1 ++ uploaded_files))} endNote: While client metadata cannot be trusted, max file size validations are enforced as each chunk is received when performing direct to server uploads.For more information on implementing client-side, direct-to-cloud uploads, see the External Uploads guide.","ref":"uploads.html#consume-uploaded-entries","title":"Uploads - Consume uploaded entries","type":"extras"},{"doc":"Using Gettext for internationalizationFor internationalization with gettext, the locale used within your Plug pipeline can be stored in the Plug session and restored within your LiveView mount. For example, after user signs in or preference changes, you can write the locale to the session:def put_user_session(conn, current_user) do locale = get_locale_for_user(current_user) Gettext.put_locale(MyApp.Gettext, locale) conn |&gt; put_session(:user_id, current_user.id) |&gt; put_session(:locale, locale) endThen in your LiveView mount/3, you can restore the locale:def mount(_params, %{&quot;locale&quot; =&gt; locale}, socket) do Gettext.put_locale(MyApp.Gettext, locale) {:ok socket} end","ref":"using-gettext.html","title":"Using Gettext for internationalization","type":"extras"},{"doc":"BindingsPhoenix supports DOM element bindings for client-server interaction. For example, to react to a click on a button, you would render the element:&lt;button phx-click=&quot;inc_temperature&quot;&gt;+&lt;/button&gt;Then on the server, all LiveView bindings are handled with the handle_event callback, for example:def handle_event(&quot;inc_temperature&quot;, _value, socket) do {:ok, new_temp} = Thermostat.inc_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} endBindingAttributesParamsphx-value-*Click Eventsphx-capture-click, phx-clickFocus/Blur Eventsphx-window-focus, phx-window-blur, phx-focus, phx-blurKey Eventsphx-key, phx-window-keyup, phx-window-keydown, phx-keyup, phx-keydownForm Eventsphx-auto-recover, phx-trigger-action, phx-disable-with, phx-feedback-for, phx-submit, phx-changeRate Limitingphx-throttle, phx-debounceDOM Patchingphx-updateJS Interopphx-hook","ref":"bindings.html","title":"Bindings","type":"extras"},{"doc":"The phx-click binding is used to send click events to the server. When any client event, such as a phx-click click is pushed, the value sent to the server will be chosen with the following priority:Any number of optional phx-value- prefixed attributes, such as:&lt;div phx-click=&quot;inc&quot; phx-value-myvar1=&quot;val1&quot; phx-value-myvar2=&quot;val2&quot;&gt;will send the following map of params to the server:def handle_event(&quot;inc&quot;, %{&quot;myvar1&quot; =&gt; &quot;val1&quot;, &quot;myvar2&quot; =&gt; &quot;val2&quot;}, socket) doIf the phx-value- prefix is used, the server payload will also contain a &quot;value&quot; if the element's value attribute exists.When receiving a map on the server, the payload will also include user defined metadata of the client event, or an empty map if none is set. For example, the following LiveSocket client option would send the coordinates and altKey information for all clicks:let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { params: {_csrf_token: csrfToken}, metadata: { click: (e, el) =&gt; { return { altKey: e.altKey, clientX: e.clientX, clientY: e.clientY } } } })The phx-capture-click event is just like phx-click, but instead of the click event being dispatched to the closest phx-click element as it bubbles up through the DOM, the event is dispatched as it propagates from the top of the DOM tree down to the target element. This is useful when wanting to bind click events without receiving bubbled events from child UI elements. Since capturing happens before bubbling, this can also be important for preparing or preventing behaviour that will be applied during the bubbling phase.","ref":"bindings.html#click-events","title":"Bindings - Click Events","type":"extras"},{"doc":"Focus and blur events may be bound to DOM elements that emit such events, using the phx-blur, and phx-focus bindings, for example:&lt;input name=&quot;email&quot; phx-focus=&quot;myfocus&quot; phx-blur=&quot;myblur&quot;/&gt;To detect when the page itself has received focus or blur, phx-window-focus and phx-window-blur may be specified. These window level events may also be necessary if the element in consideration (most often a div with no tabindex) cannot receive focus. Like other bindings, phx-value-* can be provided on the bound element, and those values will be sent as part of the payload. For example:&lt;div class=&quot;container&quot; phx-window-focus=&quot;page-active&quot; phx-window-blur=&quot;page-inactive&quot; phx-value-page=&quot;123&quot;&gt; ... &lt;/div&gt;The following window-level bindings are supported:phx-window-focusphx-window-blurphx-window-keydownphx-window-keyup","ref":"bindings.html#focus-and-blur-events","title":"Bindings - Focus and Blur Events","type":"extras"},{"doc":"The onkeydown, and onkeyup events are supported via the phx-keydown, and phx-keyup bindings. Each binding supports a phx-key attribute, which triggers the event for the specific key press. If no phx-key is provided, the event is triggered for any key press. When pushed, the value sent to the server will contain the &quot;key&quot; that was pressed, plus any user-defined metadata. For example, pressing the Escape key looks like this:%{&quot;key&quot; =&gt; &quot;Escape&quot;}To capture additional user-defined metadata, the metadata option for keydown events may be provided to the LiveSocket constructor. For example:let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { params: {_csrf_token: csrfToken}, metadata: { keydown: (e, el) =&gt; { return { key: e.key, metaKey: e.metaKey, repeat: e.repeat } } } })To determine which key has been pressed you should use key value. The available options can be found on MDN or via the Key Event Viewer.By default, the bound element will be the event listener, but a window-level binding may be provided via phx-window-keydown or phx-window-keyup, for example:def render(assigns) do ~L&quot;&quot;&quot; &lt;div id=&quot;thermostat&quot; phx-window-keyup=&quot;update_temp&quot;&gt; Current temperature: &lt;%= @temperature %&gt; &lt;/div&gt; &quot;&quot;&quot; end def handle_event(&quot;update_temp&quot;, %{&quot;key&quot; =&gt; &quot;ArrowUp&quot;}, socket) do {:ok, new_temp} = Thermostat.inc_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} end def handle_event(&quot;update_temp&quot;, %{&quot;key&quot; =&gt; &quot;ArrowDown&quot;}, socket) do {:ok, new_temp} = Thermostat.dec_temperature(socket.assigns.id) {:noreply, assign(socket, :temperature, new_temp)} end def handle_event(&quot;update_temp&quot;, _key, socket) do {:noreply, socket} end","ref":"bindings.html#key-events","title":"Bindings - Key Events","type":"extras"},{"doc":"All events can be rate-limited on the client by using the phx-debounce and phx-throttle bindings, with the following behavior:phx-debounce - Accepts either an integer timeout value (in milliseconds), or &quot;blur&quot;. When an integer is provided, emitting the event is delayed by the specified milliseconds. When &quot;blur&quot; is provided, emitting the event is delayed until the field is blurred by the user. Debouncing is typically used for input elements.phx-throttle - Accepts an integer timeout value to throttle the event in milliseconds. Unlike debounce, throttle will immediately emit the event, then rate limit it at once per provided timeout. Throttling is typically used to rate limit clicks, mouse and keyboard actions.For example, to avoid validating an email until the field is blurred, while validating the username at most every 2 seconds after a user changes the field:&lt;form phx-change=&quot;validate&quot; phx-submit=&quot;save&quot;&gt; &lt;input type=&quot;text&quot; name=&quot;user[email]&quot; phx-debounce=&quot;blur&quot;/&gt; &lt;input type=&quot;text&quot; name=&quot;user[username]&quot; phx-debounce=&quot;2000&quot;/&gt; &lt;/form&gt;And to rate limit a volume up click to once every second:&lt;button phx-click=&quot;volume_up&quot; phx-throttle=&quot;1000&quot;&gt;+&lt;/button&gt;Likewise, you may throttle held-down keydown:&lt;div phx-window-keydown=&quot;keydown&quot; phx-throttle=&quot;500&quot;&gt; ... &lt;/div&gt;Unless held-down keys are required, a better approach is generally to use phx-keyup bindings which only trigger on key up, thereby being self-limiting. However, phx-keydown is useful for games and other use cases where a constant press on a key is desired. In such cases, throttle should always be used.Debounce and Throttle special behaviorThe following specialized behavior is performed for forms and keydown bindings:When a phx-submit, or a phx-change for a different input is triggered, any current debounce or throttle timers are reset for existing inputs.A phx-keydown binding is only throttled for key repeats. Unique keypresses back-to-back will dispatch the pressed key events.","ref":"bindings.html#rate-limiting-events-with-debounce-and-throttle","title":"Bindings - Rate limiting events with Debounce and Throttle","type":"extras"},{"doc":"The lv: event prefix supports LiveView specific features that are handled by LiveView without calling the user's handle_event/3 callbacks. Today, the following events are supported:lv:clear-flash – clears the flash when sent to the server. If a phx-value-key is provided, the specific key will be removed from the flash.For example:&lt;p class=&quot;alert&quot; phx-click=&quot;lv:clear-flash&quot; phx-value-key=&quot;info&quot;&gt; &lt;%= live_flash(@flash, :info) %&gt; &lt;/p&gt;","ref":"bindings.html#liveview-specific-events","title":"Bindings - LiveView Specific Events","type":"extras"},{"doc":"Form bindings","ref":"form-bindings.html","title":"Form bindings","type":"extras"},{"doc":"To handle form changes and submissions, use the phx-change and phx-submit events. In general, it is preferred to handle input changes at the form level, where all form fields are passed to the LiveView's callback given any single input change. For example, to handle real-time form validation and saving, your template would use both phx_change and phx_submit bindings:&lt;%= f = form_for @changeset, &quot;#&quot;, [phx_change: :validate, phx_submit: :save] %&gt; &lt;%= label f, :username %&gt; &lt;%= text_input f, :username %&gt; &lt;%= error_tag f, :username %&gt; &lt;%= label f, :email %&gt; &lt;%= text_input f, :email %&gt; &lt;%= error_tag f, :email %&gt; &lt;%= submit &quot;Save&quot; %&gt; &lt;/form&gt;Reminder: form_for/3 is a Phoenix.HTML helper. Don't forget to include use Phoenix.HTML at the top of your LiveView, if using Phoenix.HTML helpers. Also, if using error_tag/2, don't forget to import MyAppWeb.ErrorHelpers.Next, your LiveView picks up the events in handle_event callbacks:def render(assigns) ... def mount(_params, _session, socket) do {:ok, assign(socket, %{changeset: Accounts.change_user(%User{})})} end def handle_event(&quot;validate&quot;, %{&quot;user&quot; =&gt; params}, socket) do changeset = %User{} |&gt; Accounts.change_user(params) |&gt; Map.put(:action, :insert) {:noreply, assign(socket, changeset: changeset)} end def handle_event(&quot;save&quot;, %{&quot;user&quot; =&gt; user_params}, socket) do case Accounts.create_user(user_params) do {:ok, user} -&gt; {:noreply, socket |&gt; put_flash(:info, &quot;user created&quot;) |&gt; redirect(to: Routes.user_path(MyAppWeb.Endpoint, MyAppWeb.User.ShowView, user))} {:error, %Ecto.Changeset{} = changeset} -&gt; {:noreply, assign(socket, changeset: changeset)} end endThe validate callback simply updates the changeset based on all form input values, then assigns the new changeset to the socket. If the changeset changes, such as generating new errors, render/1 is invoked and the form is re-rendered.Likewise for phx-submit bindings, the same callback is invoked and persistence is attempted. On success, a :noreply tuple is returned and the socket is annotated for redirect with Phoenix.LiveView.redirect/2 to the new user page, otherwise the socket assigns are updated with the errored changeset to be re-rendered for the client.","ref":"form-bindings.html#form-events","title":"Form bindings - Form Events","type":"extras"},{"doc":"For proper form error tag updates, the error tag must specify which input it belongs to. This is accomplished with the phx-feedback-for attribute, which specifies the name (or id, for backwards compatibility) of the input it belongs to. Failing to add the phx-feedback-for attribute will result in displaying error messages for form fields that the user has not changed yet (e.g. required fields further down on the page).For example, your MyAppWeb.ErrorHelpers may use this function:def error_tag(form, field) do form.errors |&gt; Keyword.get_values(field) |&gt; Enum.map(fn error -&gt; content_tag(:span, translate_error(error), class: &quot;invalid-feedback&quot;, phx_feedback_for: input_name(form, field) ) end) endNow, any DOM container with the phx-feedback-for attribute will receive a phx-no-feedback class in cases where the form fields has yet to receive user input/focus. The following css rules are generated in new projects to hide the errors:.phx-no-feedback.invalid-feedback, .phx-no-feedback .invalid-feedback { display: none; }","ref":"form-bindings.html#phx-feedback-for","title":"Form bindings - phx-feedback-for","type":"extras"},{"doc":"Number inputs are a special case in LiveView forms. On programmatic updates, some browsers will clear invalid inputs. So LiveView will not send change events from the client when an input is invalid, instead allowing the browser's native validation UI to drive user interaction. Once the input becomes valid, change and submit events will be sent normally.&lt;input type=&quot;number&quot;&gt;This is known to have a plethora of problems including accessibility, large numbers are converted to exponential notation and scrolling can accidentally increase or decrease the number.As of early 2020, the following avoids these pitfalls and will likely serve your application's needs and users much better. According to https://caniuse.com/#search=inputmode, the following is supported by 90% of the global mobile market with Firefox yet to implement.&lt;input type=&quot;text&quot; inputmode=&quot;numeric&quot; pattern=&quot;[0-9]*&quot;&gt;","ref":"form-bindings.html#number-inputs","title":"Form bindings - Number inputs","type":"extras"},{"doc":"Password inputs are also special cased in Phoenix.HTML. For security reasons, password field values are not reused when rendering a password input tag. This requires explicitly setting the :value in your markup, for example:&lt;%= password_input f, :password, value: input_value(f, :password) %&gt; &lt;%= password_input f, :password_confirmation, value: input_value(f, :password_confirmation) %&gt; &lt;%= error_tag f, :password %&gt; &lt;%= error_tag f, :password_confirmation %&gt;","ref":"form-bindings.html#password-inputs","title":"Form bindings - Password inputs","type":"extras"},{"doc":"LiveView forms support reactive file inputs, including drag and drop support via the phx-drop-target attribute:&lt;div class=&quot;container&quot; phx-drop-target=&quot;&lt;%= @uploads.avatar.ref %&gt;&quot;&gt; ... &lt;%= live_file_input @uploads.avatar %&gt; &lt;/div&gt;See Phoenix.LiveView.Helpers.live_file_input/2 for more.","ref":"form-bindings.html#file-inputs","title":"Form bindings - File inputs","type":"extras"},{"doc":"The phx-trigger-action attribute can be added to a form to trigger a standard form submit on DOM patch to the URL specified in the form's standard action attribute. This is useful to perform pre-final validation of a LiveView form submit before posting to a controller route for operations that require Plug session mutation. For example, in your LiveView template you can annotate the phx-trigger-action with a boolean assign:&lt;%= f = form_for @changeset, Routes.reset_password_path(@socket, :create), phx_submit: :save, phx_trigger_action: @trigger_submit %&gt;Then in your LiveView, you can toggle the assign to trigger the form with the current fields on next render:def handle_event(&quot;save&quot;, params, socket) do case validate_change_password(socket.assigns.user, params) do {:ok, changeset} -&gt; {:noreply, assign(socket, changeset: changeset, trigger_submit: true)} {:error, changeset} -&gt; {:noreply, assign(socket, changeset: changeset)} end endOnce phx-trigger-action is true, LiveView disconnects and then submits the form.","ref":"form-bindings.html#submitting-the-form-action-over-http","title":"Form bindings - Submitting the form action over HTTP","type":"extras"},{"doc":"By default, all forms marked with phx-change will recover input values automatically after the user has reconnected or the LiveView has remounted after a crash. This is achieved by the client triggering the same phx-change to the server as soon as the mount has been completed.Note: if you want to see form recovery working in development, please make sure to disable live reloading in development by commenting out the LiveReload plug in your endpoint.ex file or by setting code_reloader: false in your config/dev.exs. Otherwise live reloading may cause the current page to be reloaded whenever you restart the server, which will discard all form state.For most use cases, this is all you need and form recovery will happen without consideration. In some cases, where forms are built step-by-step in a stateful fashion, it may require extra recovery handling on the server outside of your existing phx-change callback code. To enable specialized recovery, provide a phx-auto-recover binding on the form to specify a different event to trigger for recovery, which will receive the form params as usual. For example, imagine a LiveView wizard form where the form is stateful and built based on what step the user is on and by prior selections:&lt;form phx-change=&quot;validate_wizard_step&quot; phx-auto-recover=&quot;recover_wizard&quot;&gt;On the server, the &quot;validate_wizard_step&quot; event is only concerned with the current client form data, but the server maintains the entire state of the wizard. To recover in this scenario, you can specify a recovery event, such as &quot;recover_wizard&quot; above, which would wire up to the following server callbacks in your LiveView:def handle_event(&quot;validate_wizard_step&quot;, params, socket) do # regular validations for current step {:noreply, socket} end def handle_event(&quot;recover_wizard&quot;, params, socket) do # rebuild state based on client input data up to the current step {:noreply, socket} endTo forgo automatic form recovery, set phx-auto-recover=&quot;ignore&quot;.","ref":"form-bindings.html#recovery-following-crashes-or-disconnects","title":"Form bindings - Recovery following crashes or disconnects","type":"extras"},{"doc":"The JavaScript client is always the source of truth for current input values. For any given input with focus, LiveView will never overwrite the input's current value, even if it deviates from the server's rendered updates. This works well for updates where major side effects are not expected, such as form validation errors, or additive UX around the user's input values as they fill out a form.For these use cases, the phx-change input does not concern itself with disabling input editing while an event to the server is in flight. When a phx-change event is sent to the server, the input tag and parent form tag receive the phx-change-loading css class, then the payload is pushed to the server with a &quot;_target&quot; param in the root payload containing the keyspace of the input name which triggered the change event.For example, if the following input triggered a change event:&lt;input name=&quot;user[username]&quot;/&gt;The server's handle_event/3 would receive a payload:%{&quot;_target&quot; =&gt; [&quot;user&quot;, &quot;username&quot;], &quot;user&quot; =&gt; %{&quot;username&quot; =&gt; &quot;Name&quot;}}The phx-submit event is used for form submissions where major side effects typically happen, such as rendering new containers, calling an external service, or redirecting to a new page.On submission of a form bound with a phx-submit event:The form's inputs are set to readonlyAny submit button on the form is disabledThe form receives the &quot;phx-submit-loading&quot; classOn completion of server processing of the phx-submit event:The submitted form is reactivated and loses the &quot;phx-submit-loading&quot; classThe last input with focus is restored (unless another input has received focus)Updates are patched to the DOM as usualTo handle latent events, any HTML tag can be annotated with phx-disable-with, which swaps the element's innerText with the provided value during event submission. For example, the following code would change the &quot;Save&quot; button to &quot;Saving...&quot;, and restore it to &quot;Save&quot; on acknowledgment:&lt;button type=&quot;submit&quot; phx-disable-with=&quot;Saving...&quot;&gt;Save&lt;/button&gt;You may also take advantage of LiveView's CSS loading state classes to swap out your form content while the form is submitting. For example, with the following rules in your app.css:.while-submitting { display: none; } .inputs { display: block; } .phx-submit-loading { .while-submitting { display: block; } .inputs { display: none; } }You can show and hide content with the following markup:&lt;form phx-change=&quot;update&quot;&gt; &lt;div class=&quot;while-submitting&quot;&gt;Please wait while we save our content...&lt;/div&gt; &lt;div class=&quot;inputs&quot;&gt; &lt;input type=&quot;text&quot; name=&quot;text&quot; value=&quot;&lt;%= @text %&gt;&quot;&gt; &lt;/div&gt; &lt;/form&gt;Additionally, we strongly recommend including a unique HTML &quot;id&quot; attribute on the form. When DOM siblings change, elements without an ID will be replaced rather than moved, which can cause issues such as form fields losing focus.","ref":"form-bindings.html#javascript-client-specifics","title":"Form bindings - JavaScript client specifics","type":"extras"},{"doc":"DOM patching and temporary assignsA container can be marked with phx-update, allowing the DOM patch operations to avoid updating or removing portions of the LiveView, or to append or prepend the updates rather than replacing the existing contents. This is useful for client-side interop with existing libraries that do their own DOM operations. The following phx-update values are supported:replace - the default operation. Replaces the element with the contentsignore - ignores updates to the DOM regardless of new content changesappend - append the new DOM contents instead of replacingprepend - prepend the new DOM contents instead of replacingWhen using phx-update, a unique DOM ID must always be set in the container. If using &quot;append&quot; or &quot;prepend&quot;, a DOM ID must also be set for each child. When appending or prepending elements containing an ID already present in the container, LiveView will replace the existing element with the new content instead appending or prepending a new element.The &quot;ignore&quot; behaviour is frequently used when you need to integrate with another JS library. The &quot;append&quot; and &quot;prepend&quot; feature is often used with &quot;Temporary assigns&quot; to work with large amounts of data. Let's learn more.","ref":"dom-patching.html","title":"DOM patching and temporary assigns","type":"extras"},{"doc":"By default, all LiveView assigns are stateful, which enables change tracking and stateful interactions. In some cases, it's useful to mark assigns as temporary, meaning they will be reset to a default value after each update. This allows otherwise large but infrequently updated values to be discarded after the client has been patched.Imagine you want to implement a chat application with LiveView. You could render each message like this:&lt;%= for message &lt;- @messages do %&gt; &lt;p&gt;&lt;span&gt;&lt;%= message.username %&gt;:&lt;/span&gt; &lt;%= message.text %&gt;&lt;/p&gt; &lt;% end %&gt;Every time there is a new message, you would append it to the @messages assign and re-render all messages.As you may suspect, keeping the whole chat conversation in memory and resending it on every update would be too expensive, even with LiveView smart change tracking. By using temporary assigns and phx-update, we don't need to keep any messages in memory, and send messages to be appended to the UI only when there are new ones.To do so, the first step is to mark which assigns are temporary and what values they should be reset to on mount:def mount(_params, _session, socket) do socket = assign(socket, :messages, load_last_20_messages()) {:ok, socket, temporary_assigns: [messages: []]} endOn mount we also load the initial number of messages we want to send. After the initial render, the initial batch of messages will be reset back to an empty list.Now, whenever there are one or more new messages, we will assign only the new messages to @messages:socket = assign(socket, :messages, new_messages)In the template, we want to wrap all of the messages in a container and tag this content with phx-update. Remember, we must add an ID to the container as well as to each child:&lt;div id=&quot;chat-messages&quot; phx-update=&quot;append&quot;&gt; &lt;%= for message &lt;- @messages do %&gt; &lt;p id=&quot;&lt;%= message.id %&gt;&quot;&gt; &lt;span&gt;&lt;%= message.username %&gt;:&lt;/span&gt; &lt;%= message.text %&gt; &lt;/p&gt; &lt;% end %&gt; &lt;/div&gt;When the client receives new messages, it now knows to append to the old content rather than replace it.","ref":"dom-patching.html#temporary-assigns","title":"DOM patching and temporary assigns - Temporary assigns","type":"extras"},{"doc":"Temporary assigns are useful when you want to render some data and then discard it so LiveView no longer needs to keep it in memory.For this reason, a temporary assign is not re-rendered until it is set again. This means that temporary assigns should not be used to reset or control UI state. Let's see an example.Imagine you want to show an error message when the input is less than 3 chars. You can write this code: def render(assigns) do ~L&quot;&quot;&quot; &lt;%= if @too_short do %&gt; Input too short... &lt;% end %&gt; Searched for: &lt;%= @search %&gt; &lt;form&gt;&lt;input phx-change=&quot;search&quot; name=&quot;term&quot; /&gt;&lt;/form&gt; &quot;&quot;&quot; end def mount(_params, _session, socket) do {:ok, assign(socket, too_short: false, search: &quot;&quot;), temporary_assigns: [too_short: false]} end def handle_event(&quot;search&quot;, %{&quot;term&quot; =&gt; term}, socket) do # do not search if user provides less then 3 chars if String.length(term) &gt;= 3 do {:noreply, assign(socket, search: term)} else {:noreply, assign(socket, too_short: true, search: term)} end endThe idea here is that, while the term is less than 3 characters, we will set @too_short to true and show an error message in the UI accordingly. We also set @too_short as a temporary assign, so that it resets to false after every render.However, once a temporary assign resets to its original value, it won't be re-rendered, unless we explicitly assign it to something else. This means that the LiveView will never re-render the if block and we will continue to show &quot;Input too short...&quot; even after the input has 3 or more characters.The mistake here is using :temporary_assigns to reset or control UI state, while :temporary_assigns should rather be used when we don't have (or don't want to keep) certain data around. The fix is to set too_short: false on the if branch, making sure it is reset whenever the search input changes.","ref":"dom-patching.html#pitfall-temporary-assigns-to-reset-or-control-ui-state","title":"DOM patching and temporary assigns - Pitfall: temporary assigns to reset or control UI state","type":"extras"},{"doc":"JavaScript interoperabilityAs seen earlier, you start by instantiating a single LiveSocket to enable LiveView client/server interaction, for example:import {Socket} from &quot;phoenix&quot; import LiveSocket from &quot;phoenix_live_view&quot; let csrfToken = document.querySelector(&quot;meta[name=&#39;csrf-token&#39;]&quot;).getAttribute(&quot;content&quot;) let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {params: {_csrf_token: csrfToken}}) liveSocket.connect()All options are passed directly to the Phoenix.Socket constructor, except for the following LiveView specific options:bindingPrefix - the prefix to use for phoenix bindings. Defaults &quot;phx-&quot;params - the connect_params to pass to the view's mount callback. May be a literal object or closure returning an object. When a closure is provided, the function receives the view's phx-view name.hooks – a reference to a user-defined hooks namespace, containing client callbacks for server/client interop. See the interop section below for details.","ref":"js-interop.html","title":"JavaScript interoperability","type":"extras"},{"doc":"To aid debugging on the client when troubleshooting issues, the enableDebug() and disableDebug() functions are exposed on the LiveSocket JavaScript instance. Calling enableDebug() turns on debug logging which includes LiveView life-cycle and payload events as they come and go from client to server. In practice, you can expose your instance on window for quick access in the browser's web console, for example:// app.js let liveSocket = new LiveSocket(...) liveSocket.connect() window.liveSocket = liveSocket // in the browser&#39;s web console &gt;&gt; liveSocket.enableDebug()The debug state uses the browser's built-in sessionStorage, so it will remain in effect for as long as your browser session lasts.","ref":"js-interop.html#debugging-client-events","title":"JavaScript interoperability - Debugging Client Events","type":"extras"},{"doc":"Proper handling of latency is critical for good UX. LiveView's CSS loading states allow the client to provide user feedback while awaiting a server response. In development, near zero latency on localhost does not allow latency to be easily represented or tested, so LiveView includes a latency simulator with the JavaScript client to ensure your application provides a pleasant experience. Like the enableDebug() function above, the LiveSocket instance includes enableLatencySim(milliseconds) and disableLatencySim() functions which apply throughout the current browser session. The enableLatencySim function accepts an integer in milliseconds for the round-trip-time to the server. For example:// app.js let liveSocket = new LiveSocket(...) liveSocket.connect() window.liveSocket = liveSocket // in the browser&#39;s web console &gt;&gt; liveSocket.enableLatencySim(1000) [Log] latency simulator enabled for the duration of this browser session. Call disableLatencySim() to disable","ref":"js-interop.html#simulating-latency","title":"JavaScript interoperability - Simulating Latency","type":"extras"},{"doc":"By default, the following classes are applied to the LiveView's parent container:&quot;phx-connected&quot; - applied when the view has connected to the server&quot;phx-disconnected&quot; - applied when the view is not connected to the server&quot;phx-error&quot; - applied when an error occurs on the server. Note, this class will be applied in conjunction with &quot;phx-disconnected&quot; if connection to the server is lost.All phx- event bindings apply their own css classes when pushed. For example the following markup:&lt;button phx-click=&quot;clicked&quot; phx-window-keydown=&quot;key&quot;&gt;...&lt;/button&gt;On click, would receive the phx-click-loading class, and on keydown would receive the phx-keydown-loading class. The css loading classes are maintained until an acknowledgement is received on the client for the pushed event.In the case of forms, when a phx-change is sent to the server, the input element which emitted the change receives the phx-change-loading class, along with the parent form tag. The following events receive css loading classes:phx-click - phx-click-loadingphx-change - phx-change-loadingphx-submit - phx-submit-loadingphx-focus - phx-focus-loadingphx-blur - phx-blur-loadingphx-window-keydown - phx-keydown-loadingphx-window-keyup - phx-keyup-loadingFor live page navigation via live_redirect and live_patch, as well as form submits via phx-submit, the JavaScript events &quot;phx:page-loading-start&quot; and &quot;phx:page-loading-stop&quot; are dispatched on window. Additionally, any phx- event may dispatch page loading events by annotating the DOM element with phx-page-loading. This is useful for showing main page loading status, for example:// app.js import topbar from &quot;topbar&quot; window.addEventListener(&quot;phx:page-loading-start&quot;, info =&gt; topbar.show()) window.addEventListener(&quot;phx:page-loading-stop&quot;, info =&gt; topbar.hide())Within the callback, info.detail will be an object that contains a kind key, with a value that depends on the triggering event:&quot;redirect&quot; - the event was triggered by a redirect&quot;patch&quot; - the event was triggered by a patch&quot;initial&quot; - the event was triggered by initial page load&quot;element&quot; - the event was triggered by a phx- bound element, such as phx-clickFor all kinds of page loading events, all but &quot;element&quot; will receive an additional to key in the info metadata pointing to the href associated with the page load.In the case of an &quot;element&quot; page loading event, the info will contain a &quot;target&quot; key containing the DOM element which triggered the page loading state.","ref":"js-interop.html#loading-state-and-errors","title":"JavaScript interoperability - Loading state and errors","type":"extras"},{"doc":"To handle custom client-side JavaScript when an element is added, updated, or removed by the server, a hook object may be provided with the following life-cycle callbacks:mounted - the element has been added to the DOM and its server LiveView has finished mountingbeforeUpdate - the element is about to be updated in the DOM. Note: any call here must be synchronous as the operation cannot be deferred or cancelled.updated - the element has been updated in the DOM by the serverdestroyed - the element has been removed from the page, either by a parent update, or by the parent being removed entirelydisconnected - the element's parent LiveView has disconnected from the serverreconnected - the element's parent LiveView has reconnected to the serverThe above life-cycle callbacks have in-scope access to the following attributes:el - attribute referencing the bound DOM node,viewName - attribute matching the DOM node's phx-view valuepushEvent(event, payload, (reply, ref) =&gt; ...) - method to push an event from the client to the LiveView serverpushEventTo(selectorOrTarget, event, payload, (reply, ref) =&gt; ...) - method to push targeted events from the client to LiveViews and LiveComponents.handleEvent(event, (payload) =&gt; ...) - method to handle an event pushed from the serverFor example, the markup for a controlled input for phone-number formatting could be written like this:&lt;input type=&quot;text&quot; name=&quot;user[phone_number]&quot; id=&quot;user-phone-number&quot; phx-hook=&quot;PhoneNumber&quot; /&gt;Then a hook callback object could be defined and passed to the socket:let Hooks = {} Hooks.PhoneNumber = { mounted() { this.el.addEventListener(&quot;input&quot;, e =&gt; { let match = this.el.value.replace(/\\D/g, &quot;&quot;).match(/^(\\d{3})(\\d{3})(\\d{4})$/) if(match) { this.el.value = `${match[1]}-${match[2]}-${match[3]}` } }) } } let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, {hooks: Hooks, ...}) ...Note: when using phx-hook, a unique DOM ID must always be set.For integration with client-side libraries which require a broader access to full DOM management, the LiveSocket constructor accepts a dom option with an onBeforeElUpdated callback. The fromEl and toEl DOM nodes are passed to the function just before the DOM patch operations occurs in LiveView. This allows external libraries to (re)initialize DOM elements or copy attributes as necessary as LiveView performs its own patch operations. The update operation cannot be cancelled or deferred, and the return value is ignored. For example, the following option could be used to add Alpine.js support to your project:let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { ..., dom: { onBeforeElUpdated(from, to){ if(from.__x){ window.Alpine.clone(from.__x, to) } } }, })Client-server communicationA hook can push events to the LiveView by using the pushEvent function and receive a reply from the server via a {:reply, map, socket} return value. The reply payload will be passed to the optional pushEvent response callback.Communication with the hook from the server can be done by reading data attributes on the hook element or by using push_event on the server and handleEvent on the client.For example, to implement infinite scrolling, one can pass the current page using data attributes:&lt;div id=&quot;infinite-scroll&quot; phx-hook=&quot;InfiniteScroll&quot; data-page=&quot;&lt;%= @page %&gt;&quot;&gt;And then in the client:Hooks.InfiniteScroll = { page() { return this.el.dataset.page }, mounted(){ this.pending = this.page() window.addEventListener(&quot;scroll&quot;, e =&gt; { if(this.pending == this.page() &amp;&amp; scrollAt() &gt; 90){ this.pending = this.page() + 1 this.pushEvent(&quot;load-more&quot;, {}) } }) }, updated(){ this.pending = this.page() } }However, the data attribute approach is not a good approach if you need to frequently push data to the client. To push out-of-band events to the client, for example to render charting points, one could do:&lt;div id=&quot;chart&quot; phx-hook=&quot;Chart&quot;&gt; {:noreply, push_event(socket, &quot;points&quot;, %{points: new_points})}And then on the client:Hooks.Chart = { mounted(){ this.handleEvent(&quot;points&quot;, ({points}) =&gt; MyChartLib.addPoints(points)) } }Note: events pushed from the server via push_event are global and will be dispatched to all active hooks on the client who are handling that event.Note: In case a LiveView pushes events and renders content, handleEvent callbacks are invoked after the page is updated. Therefore, if the LiveView redirects at the same time it pushes events, callbacks won't be invoked.","ref":"js-interop.html#client-hooks","title":"JavaScript interoperability - Client hooks","type":"extras"},{"doc":"External UploadsUploads to external cloud providers, such as Amazon S3, Google Cloud, etc., can be achieved by using the :external option in allow_upload/3.You provide a 2-arity function to allow the server to generate metadata for each upload entry, which is passed to a user-specified JavaScript function on the client.Typically when your function is invoked, you will generate a presigned URL, specific to your cloud storage provider, that will provide temporary access for the end-user to upload data directly to your cloud storage.","ref":"uploads-external.html","title":"External Uploads","type":"extras"},{"doc":"For any service that supports large file uploads via chunked HTTP requests with Content-Range headers, you can use the UpChunk JS library by Mux to do all the hard work of uploading the file.You only need to wire the UpChunk instance to the LiveView UploadEntry callbacks, and LiveView will take care of the rest.Install UpChunk:npm install --prefix assets --save @mux/upchunkConfigure your uploader on Phoenix.LiveView.mount/3:def mount(_params, _session, socket) do {:ok, socket |&gt; assign(:uploaded_files, []) |&gt; allow_upload(:avatar, accept: :any, max_entries: 3, external: &amp;presign_upload/2)} endSupply the :external option to Phoenix.LiveView.allow_upload/3. It requires a 2-arity function that generates a signed URL where the client will push the bytes for the upload entry.For example, if you were using a context that provided a start_session function, you might write something like this:defp presign_upload(entry, socket) do {:ok, %{&quot;Location&quot; =&gt; link}} = SomeTube.start_session(%{ &quot;uploadType&quot; =&gt; &quot;resumable&quot;, &quot;x-upload-content-length&quot; =&gt; entry.client_size }) {:ok, %{uploader: &quot;UpChunk&quot;, entrypoint: link}, socket} endFinally, on the client-side, we use UpChunk to create an upload from the temporary URL generated on the server and attach listeners for its events to the entry's callbacks:import * as UpChunk from &quot;@mux/upchunk&quot; let Uploaders = {} Uploaders.UpChunk = function(entries, onViewError){ entries.forEach(entry =&gt; { // create the upload session with UpChunk let { file, meta: { entrypoint } } = entry let upload = UpChunk.createUpload({ entrypoint, file }) // stop uploading in the event of a view error onViewError(() =&gt; upload.pause()) // upload error triggers LiveView error upload.on(&quot;error&quot;, (e) =&gt; entry.error(e.detail.message)) // notify progress events to LiveView upload.on(&quot;progress&quot;, (e) =&gt; entry.progress(e.detail)) }) } // Don't forget to assign Uploaders to the liveSocket let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { uploaders: Uploaders, params: {_csrf_token: csrfToken} })","ref":"uploads-external.html#chunked-http-uploads","title":"External Uploads - Chunked HTTP Uploads","type":"extras"},{"doc":"In order to enforce all of your file constraints when uploading to S3, it is necessary to perform a multipart form POST with your file data.def mount(_params, _session, socket) do {:ok, socket |&gt; assign(:uploaded_files, []) |&gt; allow_upload(:avatar, accept: :any, max_entries: 3, external: &amp;presign_upload/2)} end defp presign_upload(entry, socket) do uploads = socket.assigns.uploads bucket = &quot;phx-upload-example&quot; key = &quot;public/\#{entry.client_name}&quot; config = %{ region: &quot;us-east-1&quot;, access_key_id: System.fetch_env!(&quot;AWS_ACCESS_KEY_ID&quot;), secret_access_key: System.fetch_env!(&quot;AWS_SECRET_ACCESS_KEY&quot;) } {:ok, fields} = S3.sign_form_upload(config, bucket, key: key, content_type: entry.client_type, max_file_size: uploads.avatar.max_file_size, expires_in: :timer.hours(1) ) meta = %{uploader: &quot;S3&quot;, key: key, url: &quot;http://\#{bucket}.s3.amazonaws.com&quot;, fields: fields} {:ok, meta, socket} endHere, we implemented a presign_upload/2 function, which we passed as a captured anonymous function to :external. Next, we used ExAws.S3 to generate a presigned URL for the upload. Lastly, we return our :ok result, with a payload of metadata for the client, along with our unchanged socket. The metadata must contain the :uploader key, specifying the name of the JavaScript client-side uploader, in this case &quot;S3&quot;.To complete the flow, we can implement our S3 client uploader and tell the LiveSocket where to find it:let Uploaders = {} Uploaders.S3 = function(entries, onViewError){ entries.forEach(entry =&gt; { let formData = new FormData() let {url, fields} = entry.meta Object.entries(fields).forEach(([key, val]) =&gt; formData.append(key, val)) formData.append(&quot;file&quot;, entry.file) let xhr = new XMLHttpRequest() onViewError(() =&gt; xhr.abort()) xhr.onload = () =&gt; xhr.status === 204 || entry.error() xhr.onerror = () =&gt; entry.error() xhr.upload.addEventListener(&quot;progress&quot;, (event) =&gt; { if(event.lengthComputable){ let percent = Math.round((event.loaded / event.total) * 100) entry.progress(percent) } }) xhr.open(&quot;POST&quot;, url, true) xhr.send(formData) }) } let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { uploaders: Uploaders, params: {_csrf_token: csrfToken} })We define an Uploaders.S3 function, which receives our entries. It then performs an AJAX request for each entry, using the entry.progress(), entry.error(), and entry.done() functions to report upload events back to the LiveView. Lastly, we pass the uploaders namespace to the LiveSocket constructor to tell phoenix where to find the uploaders return within the external metadata.","ref":"uploads-external.html#direct-to-s3","title":"External Uploads - Direct to S3","type":"extras"}]